#ifndef FFT_HH
#define FFT_HH


#include "Complex.hh"

/// FFT support flags
enum fft_support_type {kFFTW, kFFTDouble};

/**  Returns information about the FFT implementation.
  *  Supported flags are:
  *    kFFTW: true if FFTW3 is used
  *    kFFTDouble: true if double precision FFTs are available
  *  @memo FFT Implementation information.
  *  @param supp Support flag.
  *  @return true if feature indicated by flag is supported
  */
bool infoFFT (fft_support_type supp);

/**  Calculate a fast Fourier transform of a complex series. The 
  *  transformed data overwrite the untransformed data. The output series 
  *  store in the usual order, e.g. 0, dF, ..., F_Ny-dF, -F_Ny, ..., -dF.
  *  @memo In-place Fourier Transform of a complex series.
  *  @param x Complex series to be transformed.
  *  @param N Number of elements in the series. N may be any number > 1.
  *  @param Dir Select between transform (1) and inverse transform (0).
  */
void wfft(fComplex x[], unsigned int N, int Dir);
void wfft(dComplex x[], unsigned int N, int Dir);

/**  Calculate a fast Fourier Transform of a real series. The output
  *  data are stored as a single-sided series from 0 to F_NY, inclusive. 
  *  The output array has N/2+1 elements and may overlap the input array.
  *  @memo Fourier Transform of a real series.
  *  @param x Real time series to be fourier transformed.
  *  @param y Complex result of the transformation.
  *  @param N Number of elements in the input series. N may be any even number.
  */
void wfft(const float x[], fComplex y[], unsigned int N);
void wfft(const double x[], dComplex y[], unsigned int N);

/**  Calculate an inverse Fourier Transform of a single sided series. The 
  *  output data are stored as a float time series. The output data may 
  *  overlap the input array.
  *  @memo Inverse fourier transform of a single-sided series.
  *  @param x Complex time series to be fourier transformed.
  *  @param y Real result of the transformation.
  *  @param N Number of elements in the series. N may be any even number.
  */
void wfft(const fComplex y[], float x[], unsigned int N);
void wfft(const dComplex y[], double x[], unsigned int N);

#endif // FFT_HH
