#ifndef CALIB_CHAN_LIST_HH
#define CALIB_CHAN_LIST_HH
#include <string>

//======================================  XML file field names
#define CALXML_CHANNEL_PARAM    "Channel"
#define CALXML_VERSION_PARAM    "Version"
#define CALXML_VERSNID_PARAM    "VersionID"
#define CALXML_COMMENT_PARAM    "Comment"
#define CALXML_START_TIME_PARAM "StartTime"
#define CALXML_CALIB_TIME_PARAM "CalibrationTime"
#define CALXML_DURATION_PARAM   "Duration"

//======================================  Frame file field names
#define CALFRAME_VERSION_NAME "CalibrationVersion"
#define CALFRAME_COMMENT_NAME "CalibrationComment"


/**  The CalibChanList class generates a set of channel names to be used for 
  *  in calibration frames. A channel of the form Xn:LSC-<chan> is specified
  *  and the derived channel names of the form #Xn:CAL-<type>_<chan># are
  *  generated.
  *  @memo Calibration frame channel name list.
  *  @author John Zweizig
  *  @version 1.0; Last Moodified: August 25, 2005
  */
class CalibChanList {
public:
    /**  Output format enumerator.
      */
    enum cal_format {
        cfmt_default,
	cfmt_xml,
	cfmt_framev1,
	cfmt_framev2
    };

    /** Construct a channel list.
      */
    CalibChanList(const std::string& chan);

    /** Destroy the channel list.
      */
    ~CalibChanList(void);

    /**  Get the Sensing function (FSeries) Channel name.
      */
    const char* getSensing(void) const;
 
    /**  Get the response function (FSeries) channel name
      */
    const char* getResponse(void) const;

    /**  Get the open loop gain (FSeries) channel name.
      */
    const char* getOLoopGain(void) const;

    /**  Get the cavity factor (alpha) Time series.
      */
    const char* getCavFac(void) const;

    /**  Get the open loop gain factor (alpha*beta) time series channel name.
      */
    const char* getOLoopFac(void) const;

    /**  Set the name of the channel to be calibrated and generate all the
      *  associated channel names.
      */
    void setChannel(const std::string& chan);

    /**  Test whether the name of the channel to be calibrated has been 
      *  specified.
      */
    bool operator!(void) const;
private:
    std::string mChannel;
    std::string mSensing;
    std::string mResponse;
    std::string mOLoopGain;
    std::string mCavFac;
    std::string mOLoopFac;
};

//======================================  Inline accessors
inline const char*
CalibChanList::getSensing(void) const {
    return mSensing.c_str();
}

inline const char*
CalibChanList::getOLoopGain(void) const {
    return mOLoopGain.c_str();
}

inline const char* 
CalibChanList::getResponse(void) const {
    return mResponse.c_str();
}

inline const char* 
CalibChanList::getCavFac(void) const {
    return mCavFac.c_str();
}

inline const char* 
CalibChanList::getOLoopFac(void) const {
    return mOLoopFac.c_str();
}

inline bool 
CalibChanList::operator!(void) const {
    return !mChannel.empty();
}

#endif  // !defined(CALIB_CHAN_LIST_HH)
