#ifndef GENERATOR_GENPARAM_HH
#define GENERATOR_GENPARAM_HH

#include <iosfwd>
#include <string>

namespace generator {

  /**  The GenParam class holds a generation parameter. The parameters
    *  may have string or numeric values, and may take random values.
    *  Random parameters are reset by calls to #sampleParameter()# according
    *  to one of the following distributions:
    *
    *  \begin{tabular}{ll}
    *  Name  & Distribution \\
    *  #step(<min>,<max>,<inc>)#   & Step from #<min># to #<max># by #<inc># \\
    *  #xstep(<min>,<max>,<fact>)# & Step from #<min># to #<max># by factors of #<fact># \\
    *  #flat(<min>,<max>)# & Flat distribution in range #<min> - <max>#\\
    *  #gauss(<mean>,<sigma>)# & Gaussian distribution with mean #<mean># and sigma #<sigma># \\
    *  #power(N,<min>,<max>)#  & ${dN \over dx} \sim x^{N}$ in range #<min> - <max># \\
    *  #exp(b,<min>,<max>)# & ${dN \over dx} \sim e^{-b}$ in range #<min> - <max>#\\
    *  \end{tabular}
    *
    *  @memo Generation parameter class.
    *  @author John Zweizig
    *  @version 1.1; Last modified April 22, 2003
    */
  class GenParam {
  public:
      /** parameter type enumerator.
       */
      enum ParType {
	kString,
	kConst,
	kDistrib
      };

      /**  Distribution type enumerator.
       */
      enum DistType { 
	kStep,
	kGeom,
	kFlat,
	kGauss,
	kPower,
	kExp
      };

  public:
      /**  Default constructor.
        *  @memo Default constructor.
	*/
      GenParam(void);

      /**  Destroy a parameter object
        *  @memo Destructor.
        */
      ~GenParam(void);

      /**  Dump the parameter type and current value. In the case of 
        *  random distributions, the distribution type and parameters
	*  are also given.
        *  @memo Dump the parameter tpe and value.
	*  @param out Standard output stream to receive dump text.
	*  @return Reference to the output stream.
        */
      std::ostream& dump(std::ostream& out) const;

      /**  Get the distribution id from the name.
        *  @memo get the named distribution ID.
	*  @param name Distribution name to look up.
	*  @return ID of named distribution name.
	*/
      static DistType getDistID(const std::string& name);

      /**  Get the name of the specified distribution ID.
        *  @memo get name distribution id.
	*  @param id Distrbution id.
	*  @return Constant pointer to ditribution name string.
        */
      static const char* getDistName(DistType id);

      /**  Get a string value of the class parameter.
        *  @memo Get parameter string value.
	*  @return Constant pointer to parameter value string.
	*/
      const char* getString(void) const;

      /**  Get a numeric value of the parameter. The value of string 
        *  parameters is converted to a numeric value.
	*  @memo Get numeric parameter value.
	*  @return Numeric value of the parameter.
        */
      double getNumeric(void) const;

      /**  Test whether Parameter has a constant value.
        *  @memo Test whether parameter is constant.
        *  @return true if parameter is constant (not a distribution).
        */
      bool isConst(void) const;

      /**  Test whether Parameter steps over specified values.
        *  @memo Test if parameter is step distribution.
	*  @return true if parameter is a atep distribution.
        */
      bool isStep(void) const;

      /**  Test whether stepping parameter is at the start (indicates 
        *  overflow occurred in last step).
	*  @memo Test if at start of step.
	*  @return true if a start of step.
        */
      bool atStart(void) const;

      /**  Generate a new value for a randomly generated parameter. If 
        *  the parameter is string or numeric constant type, the current
	*  value of the parameter is returned.
	*  @memo Generate parameter according to random distribution.
	*  @return Random  parameter value.
	*/
      double sampleParameter(void);

      /**  Specify a distribution for the random parameter. The distribution
        *  is specified in the form #"type(p1[[, p2], p3])"#.
        *  @memo Set the distribution type.
	*  @param value Distribution definition string.
        */
      void setDistribution(const std::string& value);

      /**  Assign the parameter a numeric value and set the parameter type 
        *  to #kConst#.
        *  @memo Assign parameter a numeric value.
	*  @param value Value to assign to parameter.
	*/
      void setParameter(double value);

      /**  Assign the parameter a string value and set the parameter type
        *  to #kString#.
        *  @memo Assign parameter a string value.
	*  @param value String value to assign to parameter.
	*/
      void setParameter(const std::string& value);

  private:
      ParType     mType;
      std::string mStrValue;
      double      mNumValue;
      DistType    mDistr;
      double      mDistPar[3];
  };


  //====================================  Inline methods
  inline bool 
  GenParam::isConst(void) const {
      return mType != kDistrib;
  }

  inline bool 
  GenParam::isStep(void) const {
      return !isConst() && mDistr == kStep;
  }

  inline bool 
  GenParam::atStart(void) const {
      return isStep() && (mNumValue == mDistPar[0]);
  }

  inline const char* 
  GenParam::getString(void) const {
      return mStrValue.c_str();
  }

  inline double 
  GenParam::getNumeric(void) const {
      return mNumValue;
  }

} // namespace generator

#endif  // GENERATOR_SINGAUSS_HH
