#ifndef LMSG_TRANSINPUT_HH
#define LMSG_TRANSINPUT_HH

#include <stdexcept>
#include <string>
#include "lmsg/MsgTypes.hh"

class Time;

namespace lmsg {

  class Buffer;

  /**  Message input stream with translation capability. The TransInput 
    *  class is effectively an input stream that is intelligent enough 
    *  to translate between the big-endian external format and an internal
    *  format appropriate to the execution time machine.
    *
    *  @author John Zweizig
    *  @version 1.1; Modified August 20, 2001
    */
  class TransInput {
  public:
    /**  Construct an input stream and attach a full message buffer to it.
      *  @memo data constructor.
      *  @param b Message buffer to be read by the input stream.
      */
    TransInput(const Buffer& b);

    /**  Default input stream constructor.
      */
    TransInput(void);

    /**  Test whether there is any more data in the input buffer.
      *  @memo Test whether buffer is empty;
      *  @return true if the buffer is empty.
      */
    bool empty(void) const;

    /**  Read and translate a character array.
      *  @memo read a character array.
      *  @return number of characters read.
      *  @param x Pointer to a character array to receive the data.
      *  @param N Number of characters to be read.
      */
    size_type read(char*   x, size_type N=1) throw(std::runtime_error);

    /**  Read and translate a array of strings.
      *  @memo Read character strings.
      *  @return number of strings read.
      *  @param x Pointer to array of strings to receive the data.
      *  @param N Number of strings to be read.
      */
    size_type read(std::string* x, size_type N=1) throw(std::runtime_error);

    /**  Read and translate a short integer array.
      *  @memo Read a short integer array.
      *  @return Number of short integers read.
      *  @param x Pointer to a character array to receive the data.
      *  @param N Number of characters to be read.
      */
    size_type read(short*  x, size_type N=1) throw(std::runtime_error);

    /**  Read and translate an integer array.
      *  @memo Read an integer array.
      *  @return Number of integers read.
      *  @param x Pointer to an integer array to receive the data.
      *  @param N Number of integers to be read.
      */
    size_type read(int*    x, size_type N=1) throw(std::runtime_error);

    /**  Read and translate a long integer array.
      *  @memo Read a long integer array.
      *  @return number of characters read.
      *  @param x Pointer to a long integer array to receive the data.
      *  @param N Number of long integers to be read.
      */
    size_type read(long*   x, size_type N=1) throw(std::runtime_error);

    /**  Read and translate a float array.
      *  @memo read a character array.
      *  @return Number of floats read.
      *  @param x Pointer to a float array to receive the data.
      *  @param N Number of floats to be read.
      */
    size_type read(float*  x, size_type N=1) throw(std::runtime_error);

    /**  Read and translate a double array.
      *  @memo Read a double array.
      *  @return Number of doubles read.
      *  @param x Pointer to a double array to receive the data.
      *  @param N Number of doubles to be read.
      */
    size_type read(double* x, size_type N=1) throw(std::runtime_error);

    /**  Read and translate an array of Time objects.
      *  @memo Read an array of Times.
      *  @return number of Times read.
      *  @param x Pointer to a Time array to receive the data.
      *  @param N Number of Times to be read.
      */
    size_type read(Time*   x, size_type N=1) throw(std::runtime_error);

#if __SUNPRO_CC == 0x500
    TransInput& operator>>(char& x) throw(std::runtime_error);
    TransInput& operator>>(std::string& x) throw(std::runtime_error);
    TransInput& operator>>(short& x) throw(std::runtime_error);
    TransInput& operator>>(int& x) throw(std::runtime_error);
    TransInput& operator>>(long& x) throw(std::runtime_error);
    TransInput& operator>>(float& x) throw(std::runtime_error);
    TransInput& operator>>(double& x) throw(std::runtime_error);
    TransInput& operator>>(Time& x) throw(std::runtime_error);
#else
    /** Stream input operator template;
      */
    template <class T> TransInput& operator>>(T& x) throw(std::runtime_error);
#endif
    void align(size_type bdry);
  private:
    const char* mData;
    size_type      mIndex;
    size_type      mLength;
  };  //  class lmsg::TransInput

}    // namespace lmsg

//--------------------------------------  Implement >> by template;
#if __SUNPRO_CC == 0x500
inline lmsg::TransInput&
lmsg::TransInput::operator>>(char& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}

inline lmsg::TransInput&
lmsg::TransInput::operator>>(std::string& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}

inline lmsg::TransInput&
lmsg::TransInput::operator>>(short& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}

inline lmsg::TransInput&
lmsg::TransInput::operator>>(int& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}

inline lmsg::TransInput&
lmsg::TransInput::operator>>(long& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}

inline lmsg::TransInput&
lmsg::TransInput::operator>>(float& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}

inline lmsg::TransInput&
lmsg::TransInput::operator>>(double& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}

inline lmsg::TransInput&
lmsg::TransInput::operator>>(Time& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}
#else
template<class T>
inline lmsg::TransInput&
lmsg::TransInput::operator>> (T& x) throw(std::runtime_error) {
    read(&x);
    return *this;
}
#endif

inline bool
lmsg::TransInput::empty(void) const {
    return mIndex >= mLength;
}

#endif  //  LMSG_TRANSINPUT_HH
