#ifndef LSMP_INT_HH
#define LSMP_INT_HH
//
//   Ligo Shared Memory Partition internal structures.
//
#include <sys/types.h>
#include "ManyFlags.hh"
#include "lsmp.hh"

#define LSMP_VERSION 101                    // Package version number
#define LSMP_SHMBASE 32                     // shm base id
#define LSMP_MAXSHM   8                     // maximum # of shm partitions
#define LSMP_MAXCONS  32                    // maximum number of consumers
#define LSMP_CONSPWD  8                     // consumers per mask word
#define LSMP_LMASK   ((LSMP_MAXCONS-1)/LSMP_CONSPWD+1) // consumer mask length
#define CON_BIT(i)  (i%LSMP_CONSPWD)
#define CON_MASK(i) (1 << (i%LSMP_CONSPWD))
#define CON_WORD(i) (i/LSMP_CONSPWD)

//--------------------------------------  Consumer flag class
#define LSMP_CONSPWD  8
#define LSMP_LMASK   ((LSMP_MAXCONS-1)/LSMP_CONSPWD+1)
#define CON_BIT(i)  (i%LSMP_CONSPWD)
#define CON_MASK(i) (1 << (i%LSMP_CONSPWD))
#define CON_WORD(i) (i/LSMP_CONSPWD)

typedef ManyFlags<LSMP_LMASK,LSMP_CONSPWD> ConFlags;

//---------------------------------------  Buffer header structure
#define SEG_QUEUED 0x0001

struct LSMP_buffer {
    int      status;
    int      boff;
    ConFlags reserve_mask;

    /**  Bit mask containing one bit per consumer and is used to prevent 
      *  a buffer. The seen_mask bit for a given consumer is set in 
      *  LSMP_CONS::get_buffer() when the buffer is allocated to a consumer 
      *  or when it is skipped by the setNSkip() mechanism. The entire 
      *  seen_mask is cleared when a buffer is returned or released (for 
      *  distribution) by a producer.
      *  @memo Mask of consumers that have seen the buffer.
      */
    ConFlags seen_mask;

    /**  The use count indicates the number of consumers that have reserved
      *  or allocated the buffer. The use count is zeroed when the partition 
      *  is constructed or when the buffer is released or returned by a 
      *  producer. The count is decremented when a consumer frees the buffer 
      *  or a consumer to which the buffer is allocated is deleted, and it 
      *  is incremented when buffer is allocated by a consumer.
      *  @memo Number of consumers reserving this buffer.
      */
    int      use_count;
    int      ldata;
    int      trig;
    time_t   fill_time;
    int      data_ID;
    pid_t    owner_pid;
    int      link;
    int      evt_count;

    bool inUse(void)  const {return use_count || reserve_mask.any();}
    bool seen(void)   const {return seen_mask.any();}
    bool queued(void) const {return (status & SEG_QUEUED) != 0;}
};

//---------------------------------------  Buffer queue structure
class LSMP_BufferQ {
  public:
    int head;
    int tail;

    LSMP_BufferQ() {
        init();
    }

    void init() {
        head = -1;
	tail = -1;
    }

    bool empty() {
        return (head < 0);}

    int length(LSMP_buffer *buf) const {
        int N = 0;
	for (int i=head ; i>=0 ;i=buf[i].link) N++;
	return N;
    }

    void link(LSMP_buffer *buf, int ib) {
        buf[ib].status |= SEG_QUEUED;
        buf[ib].link = -1;
        if (head<0) head           = ib;
        else        buf[tail].link = ib;
        tail = ib;}

    int remove(LSMP_buffer *buf) {
        int i = head;
	if (i >= 0) {
	    head = buf[i].link;
	    if (head<0) tail = -1;
	    buf[i].link = -1;
	    buf[i].status &= ~SEG_QUEUED;}
	return i;}

    int remove(LSMP_buffer *buf, int ib) {
        if (ib < 0) {
	    return ib;
        } else if (head == ib) {
	    return remove(buf);
	} else {
	    for (int i=head ; i>=0 ; i=buf[i].link) {
	        if (buf[i].link == ib) {
		    buf[i].link = buf[ib].link;
		    if (buf[i].link < 0) tail = i;
		    buf[ib].link = -1;
		    buf[ib].status &= ~SEG_QUEUED;
		    return ib;
		}
	    }
	}
	return -1;
    }
};

//--------------------------------------  Shared memory structure.
#define RELBUF  0x0001                     // release unrequired buffers
#define SCAVAGE 0x0002                     // search for unused buffers
#define RQSYNCH 0x0004                     // synchronize on requests
#define KEEP    0x0008                     // Keep an unaccessed partition
#define EXPOSE  0x0010                     // Expose data to all consumers

struct LSMP_global {
    /**  The status flags control the buffer allocation policy. The default
      *  policy is to allocate buffers from the free list only and to release
      *  a buffer only when it is not reserved, is not in use and has been 
      *  seen by at least one consumer. The policy modification bits are as 
      *  follows:
      *  
      *  RELBUF  A buffer is released if no consumers have reserved it, even
      *          if it has not been seen.
      *  SCAVAGE The oldest unused and unreserved buffer may be allocated 
      *          from the full queue. Unused buffers are not returned to
      *          the free queue.
      *  RQSYNCH No buffer may be allocated by a producer unless there is a 
      *          consumer waiting.
      *  KEEP    Don't delete the partition when the global access count 
      *          goes to zero.
      *  EXPOSE  All buffers will be automatically reserved by all current
      *          consumers.
      */
    int  status;                   // Status flags
    int  version;                  // LSMP version number
    int  gbl_count;                // Number of accesses
    int  gbl_semid;                // global semaphore ID
    int  nbuf;                     // Number of buffers
    int  lbuf;                     // Buffer length
    LSMP_BufferQ full;             // Full buffer queue
    LSMP_BufferQ free;             // Free buffer queue
    char name[LSMP_LNAME];         // Partition name
    int  ncons;                    // Number of active consumers
    int  con_semid[LSMP_LMASK];    // Consumer semaphore IDs
    ConFlags conmask;              // Consumer use mask
};



//---------------------------------------  Global semaphores
enum gblsem {
    gbl_gate,
    gbl_empty,
    gbl_full,
    gbl_synch,
    gbl_nsems};

//---------------------------------------  Consumer status structure
struct LSMP_consbk {
    int   mxbuf;                   // Maxmimum buffers consumer may request
    int   trig_mask;               // Trigger mask
    int   min_time;                // Minimum time between events
    int   min_sep;                 // minimum separation between events
    int   time_ctr;       
    int   skip_ctr;

    /**  Number of segments currently allocated to the consumer. The number
      *  of allocated segments is the number of allocated or reserved 
      *  segments. This counter is incremented when a buffer is reserved
      *  (in LSMP_PROD::distribute) or allocated (LSMP_CON::get_buffer) and
      *  decremented when the buffer is freed (LSMP_CON::free_buffer).
      */
    int   seg_ctr;
    int   seg_tot;                 // Segments read since consumer allocation
    int   flags;                   // Consumer status flags
    pid_t pid;                     // Consumer process identifier

    bool isReadAll(void) const {return (flags & READALL) != 0;}
    void clrWait(void) {flags &= ~EVWAIT;}
    void setWait(void) {flags |= EVWAIT;}
};

//--------------------------------------  Define the semctl union
//  Note: This is now necessary on most systems in use by LIGO: sun, linux 
//        and hpux. Other systems (e.g. AIX and Irix) may choke on this, 
//        but for now it's easier to leave it in unconditionally.
union semun {
      int val;
      struct semid_ds *buf;
      unsigned short *array;
};

#endif
