//
//   Shared memory stream class
//
//   Compile flags defined:
//      INTRDBG    Print status to cerr on failure of getBuffer()
//
#include <time.h>
#include "iSMbuf.hh"
#include "lsmp_con.hh"
#include <stdio.h>
#include <iostream>
#include <errno.h>

using namespace std;

//--------------------------------------  Constructors.
iSMbuf::iSMbuf(void) 
  : mConsumer(0), mBuffer(0), mLength(0)
{
#ifdef ISMBUF_TRACE
    mTrace.tracepoint("iSMbuf()", 0, 0);
#endif
}

#ifdef __GNU_STDC_OLD
iSMbuf::iSMbuf(const char *partition, ios::open_mode mode) 
#else
iSMbuf::iSMbuf(const char *partition, ios::openmode mode) 
#endif
  : mConsumer(0), mBuffer(0), mLength(0)
{
#ifdef ISMBUF_TRACE
    mTrace.tracepoint("iSMbuf(const char*, ios::openmode)", 0, 0);
#endif
    open(partition, mode);
}

#ifdef __GNU_STDC_OLD
iSMbuf* iSMbuf::open(const char *partition, ios::open_mode mode) {
#else
iSMbuf* iSMbuf::open(const char *partition, ios::openmode mode) {
#endif
#ifdef ISMBUF_TRACE
    mTrace.tracepoint("open(const char*, ios::open_mode)", 0, 0);
#endif
    if ((mode & (ios::out | ios::trunc))) return (iSMbuf*) 0;

    //----------------------------------  Create/open a new partition.
    mConsumer = new LSMP_CON(partition, 0);
    if (mConsumer->isConnected()) return this;
    delete mConsumer;
    mConsumer = 0;
    return (iSMbuf*) 0;
}

//--------------------------------------  Destructor
iSMbuf::~iSMbuf() {
    if (mConsumer) delete mConsumer;
#ifdef ISMBUF_TRACE
    mTrace.pTrace();
#endif
}

//--------------------------------------  Set the reserved buffer count
void
iSMbuf::setBCount(int nbuf) {
    if (mConsumer) mConsumer->setNBuffer(nbuf);
#ifdef ISMBUF_TRACE
    mTrace.tracepoint("setBCount(int)", nbuf, 0);
#endif
}

//--------------------------------------  Wait for a buffer of data
bool
iSMbuf::waitBuf(bool nowait) {
#ifdef ISMBUF_TRACE
    mTrace.tracepoint("waitBuf(bool)", int(nowait), 0);
#endif
    if (!mConsumer)     return false;
    if (gptr() >= egptr()) relse();

    if (mBuffer) return true;
    mBuffer = mConsumer->get_buffer(nowait ? NOWAIT : 0);
    if (mBuffer) {
        mLength = mConsumer->getLength();
	setptrs(0);
	return true;
    }
    return false;
}

//--------------------------------------  Allocate a buffer
int 
iSMbuf::doallocate() {
#ifdef ISMBUF_TRACE
    mTrace.tracepoint("doallocate()", 0, 0);
#endif
    //----------------------------------  Make sure a partition is attached.
    if (!mConsumer)          return EOF;
    if (!mConsumer->valid()) return EOF;

    //----------------------------------  Release a buffer if one is allocated
    relse();

    //----------------------------------  Get a new SM buffer.
    errno = 0;
    do {
        mBuffer = mConsumer->get_buffer();
#ifdef INTRDBG
	if (!mBuffer || !mConsumer->getLength()) {
	    perror("error in iSMbuf::doallocate()");
	    cerr << "Length is: " << mConsumer->getLength() << endl;
	}
#endif  //   def(INTRDBG)
    } while (!mBuffer && errno == EINTR);
    mLength = mConsumer->getLength();
#ifdef __GNU_STDC_OLD
    setb(const_cast<char*>(mBuffer), const_cast<char*>(mBuffer)+mLength, 1);
#endif
    return 1;
}

//--------------------------------------  Get the next data buffer
int 
iSMbuf::underflow() {
#ifdef ISMBUF_TRACE
    mTrace.tracepoint("underflow()", in_avail(), 0);
#endif

    //----------------------------------  See if data are already available
    if (in_avail() > 0) return *gptr();

    //----------------------------------  Make sure a partition is attached.
    if (!mConsumer)          return EOF;
    if (!mConsumer->valid()) return EOF;

    //----------------------------------  Release a buffer if one is allocated
    relse();

    //----------------------------------  Get a new SM buffer.
    errno = 0;
    do {
        mBuffer = mConsumer->get_buffer();
#ifdef INTRDBG
	if (!mBuffer || !mConsumer->getLength()) {
	    perror("error in iSMbuf::underflow()");
	    cerr << "Length is: " << mConsumer->getLength() << endl;
	}
#endif  // def(INTRBDG)
    } while (!mBuffer && errno == EINTR);
    if (mBuffer) {
        mLength = mConsumer->getLength();
	setptrs();
	return *mBuffer;
    } else {
        mLength = 0;
        setptrs();
        return EOF;
    }
}

//======================================  Release shared memory buffer
void
iSMbuf::relse(void) {
    if (mBuffer && mConsumer) mConsumer->free_buffer();
    mBuffer = static_cast<const char*>(0);
    mLength = 0;
    setptrs(0);
}

//======================================  Seek (but ye shant find)
#ifdef __GNU_STDC_OLD
streampos 
iSMbuf::seekoff(streamoff off, ios::seekdir dir, int mode) {
#else
iSMbuf::pos_type 
iSMbuf::seekoff(off_type off, ios_base::seekdir dir, ios_base::openmode mode){
#endif

#ifdef ISMBUF_TRACE
    mTrace.tracepoint("seekoff(streamoff, ios::seekdir, int)", off, dir);
#endif
    if (!mBuffer) underflow();
    streampos newpos(0);
    if (dir == ios::beg) {
        newpos = off;
    } else if (dir == ios::cur) {
        newpos = long(gptr()) - long(mBuffer) + off;
    } else if (dir == ios::end) {
        newpos = mLength + off;
    }
    if (newpos > mLength || newpos < 0) return (streampos) EOF;
    // setptrs(newpos);
    char* p0 = const_cast<char*>(mBuffer);
    setg(p0, p0+newpos, p0+mLength);
    return newpos;
}
 
void 
iSMbuf::setptrs(int ix) {
    char* p0 = const_cast<char*>(mBuffer);
    setg(p0, p0+ix, p0+mLength);
}

#ifdef ISMBUF_TRACE
void 
iSMbuf::trace::pTrace(void) const {
    cout << "Number of traced function calls: " << nTrace << endl;
    int npr = (nTrace < maxTrace) ? nTrace : maxTrace;
    for (int i=0 ; i < npr ; i++) {
        int inx = (nTrace - i - 1) % maxTrace;
	cout << mFuncName[inx] << " " << mArg1[inx] << " " << mArg2[inx] 
	     << endl;
    }
}
#endif
