
#ifndef XSIL_STREAM_HH
#define XSIL_STREAM_HH

#include "xsil/xobj.hh"
#include <string>
#ifdef __GNU_STDC_OLD
#include <gnusstream.h>
#else
#include <sstream>
#endif

//  If USEILWDCHAR is set the MetaTables will generate columns with
//  type=ilwd:char instead of type=lstring. The stream will also escape
//  spaces in strings.
#define USEILWDCHAR

namespace xsil {

  struct UCVec {
    UCVec(void) {}
    UCVec(const int* pN, const unsigned char* pD) : pNBytes(pN), pData(pD) {}
    const int*           pNBytes;
    const unsigned char* pData;
  };

/**  An XSIL stream may be an internal list of data or it may point to a
  *  local file or remote data.
  *  @memo XSIL data stream class.
  *  @author J. Zweizig
  *  @version 1.1; Modified December 13, 1999
  */
class Stream : public xobj {
public:

  /**  Construct an unnamed empty stream.
    *  @memo XSIL Stream default Constructor.
    */
  Stream(void);

  /**  Construct a stream. By default an empty stream is produced.
    *  @memo XSIL Stream Constructor.
    */
  Stream(const char* Name, const char* Type=0, const char* Data=0);

  /**  Stream copy constructor.
    */
  Stream(const Stream& x);

  /**  XSIL stream destructor.
    */
  ~Stream();

  /**  Stream copy constructor.
    */
  Stream& operator=(const Stream& x);

  /**  Write it out as an XML document.
    */
  void Spew(Xwriter& xout) const;

  /**  Append a string to the current local stream contents. Each striing is
    *  assumed to be a lexically complete token and is separated by the 
    *  current delimiter character from the preceding data. If the string 
    *  pushes the current line length beyond the maximum length, a new-line
    *  character is added and the white space is inserted so that the string
    *  will start at the current indent level.
    *  @memo Add a string to the current local stream.
    *  @param s Lexically complete token to be added to the stream.
    */
  void append(const std::string& s);

  /**  Create a identical copy of the current Stream.
    *  @memo Clone a stream.
    *  @return Pointer to the copy.
    */
  xobj* Clone(void) const;

  /**  Test for data in stream.
    *  @return true if the stream is empty.
    */
  bool empty(void) const;

  /**  Get the delimiter character.
    *  @memo Get delimiter.
    */
  const char* getDelimit(void) const;

  /**  Get encoding type.
    *  @memo Get encoding.
    */
  const char* getEncode(void) const;

  /**  Get object type
    */
  const char* getObjType(void) const;

  /**  Set the delimiter character.
    *  The delimiter character must be unique.
    *  @memo Set the delimiter character.
    */
  void delimit(char Delim);

  /**  Append a floating point entry to the stream string.
    *  @memo Append a float.
    *  @param x float point number to be added to the stream.
    */
  void Add(float x);

  /**  Append a string to the stream string.
    *  @memo Append a string.
    *  @param x     Character string to be appended to the stream.
    *  @param escsp Escape blanks.
    */
  void Add(const std::string& x, bool escsp=false);

  /**  Append a floating point entry to the stream string.
    *  @memo Append an integer.
    *  @param x Integer number to be added to the stream.
    */
  void Add(int x);

  /**  Append a floating point entry to the stream string.
    *  @memo Add a double float.
    *  @param x Double floating point number to be added to the stream.
    */
  void Add(double x);

  /**  Append an unsigned byte string.
    *  @memo Add a byte string.
    *  @param x reference to the byte string descriptor.
    */
  void Add(const UCVec& x);

  /**  Clear out the stream data.
    *  @memo Clear the stream data.
    */
  void Clear(void);

  /**  Fill stream from a floating point array.
    *  @memo Fill from floats.
    *  @param x float point array to be converted and copied to the stream.
    */
  void Fill(int N, const float* x, int perLine=0);

  /**  Fill stream from a double floating point array.
    *  @memo Fill from doubles.
    *  @param x double array to be converted and copied to the stream.
    */
  void Fill(int N, const double* x, int perLine=0);

  /**  Fill stream from an integer array.
    *  @memo Fill with integers.
    *  @param x Integer array to be converted and copied to the stream.
    */
  void Fill(int N, const int* x, int perLine=0);

  /**  Fill stream from a string array.
    *  @memo Fill from strings.
    *  @param x String array to be copied to the stream.
    */
  void Fill(int N, const std::string x[], int perLine=0);

  /**  Fill stream with base64 encoded data.
    *  @memo Fill from string.
    *  @param data String array to be encoded into the stream.
    *  @param N    length of data to be encoded into the stream.
    *  @return Read error if true
    */
  bool decode64Data(void* data, int dLen, int N);

  /**  Fill stream with base64 encoded data.
    *  @memo Fill from string.
    *  @param data String array to be encoded into the stream.
    *  @param N    length of data to be encoded into the stream.
    */
  void encode64Data(const char* data, int N);

  /**  Open the stream for input.
   */
  void open(void);

  /**  Insert a line break at the current position.
    *  @memo Insert a line break.
    */
  void lineBreak(void);

  bool read(double      data[], int N);
  bool read(float       data[], int N);
  bool read(int         data[], int N);
  bool read(std::string data[], int N);


  /**  Make the stream a remote stream and define the file name.
    *  @memo Define file for remote stream.
    *  @param File File name or web address from which data are to be read.
    */
  void setRemote(const char* File);

  /**  Set the input data to the specified character string.
    *  @memo set stream data.
    *  @param Data pointer to a data string.
    */
  void setData(const char* Data);

  /**  Specify the encoding technique used for binary data.
    *  @memo  Specify the encoding technique.
    *  @param Code pointer to a string describing the encodeing technique.
    */
  void setEncode(const char* Code);

  /**  Set the Maximum line size.
    *  @memo set line size.
    *  @param Size Maximum line size to be used when printing the stream.
    */
  void setSize(int Size);

private:
  /**  Expand the local stream capacity to the specified estimated length.
    */
  void estLength(std::string::size_type len);

  /**  Expand local stream capacity to allow for a current estimate of the 
    *  stream length. The estimate is arrived at by spcifying a fraction
    *  complete and dividing the current stream data length by the estimated 
    *  fraction completed.
    *  @param frac Esimated fraction of data already written to the stream.
    */
  void estDone(double frac);

private:
  std::string mData;
  std::string mRemote;
  int    mLineSz;
  int    mMaxSz;
  std::string mDelimit;
  bool   mEndLine;
  std::string mEncoding;
  std::istream* mInput;
};   // xsil::Stream

}    //  namespace xsil

//----------------------------------------  Test if stream is empty
inline bool
xsil::Stream::empty(void) const {
    return mData.empty();
}

//----------------------------------------  Get the encoding technique
inline const char*
xsil::Stream::getEncode(void) const {
    return mEncoding.c_str();
}

//----------------------------------------  Get the delimiter character
inline const char*
xsil::Stream::getDelimit(void) const {
    return mDelimit.c_str();
}

#endif  // !defined(XSIL_STREAM_HH)


