/***************************************************************************
    File        : NoiseFloorMonitor.h
    Description : Median-based noise floor tracker MNFT
 ---------------------------------------------------------------------------
    Begin       : Tue March 30 2005
    Author(s)   : Roberto Grosso
 ***************************************************************************/

#ifndef __NOISE_FLOOR_MONITOR_H
#define __NOISE_FLOOR_MONITOR_H


/**  @name NoiseFloorMonitor
  *  @memo Median-based noise floor tracker (MNFT): robust estimation of
  *  noise floor drifts in interferometric data.
  *
  *  @author Roberto Grosso; Brownsville, Texas
  *  @version 1.0; Last modified March 30, 2005
  */

// Libs
#include <iostream>
#include <sstream>
#include <fstream>
#include <list>
#include <string>
#include <vector>
#include <cstdlib>
#include <limits>
#include <algorithm>


// Project files
#include "DatEnv.hh"
#include "MonServer.hh"
#include "Interval.hh"
#include "TSeries.hh"
#include "Time.hh"
#include "Trend.hh"
#include "OperStateCondList.hh"

// HTML output
#include "html/color.hh"
#include "html/document.hh"
#include "html/table.hh"
#include "html/size.hh"
#include "html/text.hh"
#include "html/writer.hh"
#include "html/hline.hh"
#include "html/link.hh"

// NoiseFloor files
#include "Clock.h"
#include "Utilities.h"
#include "Exception.h"
#include "Numeric.h"
#include "Parameter.h"
#include "LowPassFilter.h"
#include "BandPassFilter.h"
#include "HighPassFilter.h"
#include "NotchFilter.h"
#include "WhiteningFilter.h"
#include "RunningMedian.h"
#include "FIRFilter.h"
#include "Multirate.h"
#include "Convolution.h"
#include "NormalDistribution.h"

using namespace std;

class NoiseFloorMonitor : public DatEnv, MonServer {
public:
  //! Constructor
  NoiseFloorMonitor(int argc, const char *argv[]);
  //! Destructor
  virtual ~NoiseFloorMonitor(void);

  // Methods
  //! Method called by MainLoop.
  void ProcessData(void);
  //! Handle Message signal
  void Attention(void);
private:
  //! Count the no. of time stride read.
  unsigned long mCounterTimeStrides;

  //! Length on the time stride.
  Interval mTStride;

  //! Noise floor trends.
  Trend mTrend;

  //! Current time
  Time mCurrentTime;

  //! Time stride in sec.
  double mTStrideSize;

  //! Band A noise floor.
  TSeries  mNoiseFloorBandA;

  //! Band B noise floor.
  TSeries  mNoiseFloorBandB;

  //! Band C noise floor.
  TSeries  mNoiseFloorBandC;

  //! Band D noise floor.
  TSeries  mNoiseFloorBandD;

  //! Band A noise floor, 12hs plot.
  TSeries  mNoiseFloorBandA12;

  //! Band B noise floor, 12hs plot.
  TSeries  mNoiseFloorBandB12;

  //! Band C noise floor, 12hs plot.
  TSeries  mNoiseFloorBandC12;

  //! Band D noise floor, 12hs plot.
  TSeries  mNoiseFloorBandD12;

  //! Upper threshold band A.
  TSeries mUpperThresholdBandA;

  //! Lower threshold band A.
  TSeries mLowerThresholdBandA;

  //! Upper threshold band B.
  TSeries mUpperThresholdBandB;

  //! Lower threshold band B.
  TSeries mLowerThresholdBandB;

  //! Upper threshold band C.
  TSeries mUpperThresholdBandC;

  //! Lower threshold band C.
  TSeries mLowerThresholdBandC;

  //! Upper threshold band D.
  TSeries mUpperThresholdBandD;

  //! Lower threshold band D.
  TSeries mLowerThresholdBandD;

  //! Upper 2*sigma threshold band A.
  TSeries mUpper2SigmaBandA;
  //! Lower 2*sigma threshold band A.
  TSeries mLower2SigmaBandA;
  //! Upper 2*sigma threshold band B.
  TSeries mUpper2SigmaBandB;
  //! Lower 2*sigma threshold band B.
  TSeries mLower2SigmaBandB;
  //! Upper 2*sigma threshold band C.
  TSeries mUpper2SigmaBandC;
  //! Lower 2*sigma threshold band C.
  TSeries mLower2SigmaBandC;
  //! Upper 2*sigma threshold band D.
  TSeries mUpper2SigmaBandD;
  //! Lower 2*sigma threshold band D.
  TSeries mLower2SigmaBandD;

  //! Order of the Notch Filter.
  unsigned int mOrderNotchFilter;

  //! Loops until update of whiten filter.
  unsigned int mCounterWitheFilter;

  //! Update of Whitening Filter
  unsigned int mUpdateWhiteFlt;

  //! Order of the Whitening Filter.
  unsigned int mOrderWhiteFilter;

  //! Number of frequencies in the Power Spectrum Density.
  unsigned int mNoOfFreqPsd;

  //! Size in sec. of ts. used for the computation of the PSD.
  double mSizePsd;

  //! Size is Hz. of RM filter for Whitening filter.
  double mRMSizeForWhiteningFlt;

  //! Size in sec. of the Running median filter.
  double mWindowSizeRunningMedian;

  //! Signal's sampling rate in Hz.
  double mSignalSampleRate;

  //! Processing sampling rate.
  double mSampleRate;

  //! Decimate factor for cutoff filtering 16kHz -> 4kHz.
  int mDecimateFactor;

  //! Oversampling factor.
  double mOverSamplingFactor;

  //! Filter order for low-pass and band-pass filters
  unsigned int mFltOrder;

  //! Frequency for upper cut off filtering
  double mUpperCutoffFreq;

  //! Filter order for signal bands processing.
  unsigned int mBandFltOrder;

  //! upper frequency for band A
  double mBandFreqA;

  //! upper frequency for band B
  double mBandFreqB;

  //! upper frequency for band C
  double mBandFreqC;

  //! upper frequency for band D
  double mBandFreqD;

  //! Detector Channel
  std::string mDetectorChannel;

  //! Frequencies pics (lines) for the Notch filter
  std::vector<double> mFreqNotchFilter;

  //! Frequencies bands for the Notch filter
  std::vector<double> mFreqBndWidthNotFilter;

  //! Frequency cutoff filter.
  std::vector<double> mCutoffFlt;

  //! Low-pass filter for band A.
  std::vector<double> mLpFltA;

  //! Band-pass filter for band B.
  std::vector<double> mBpFltB;

  //! Band-pass filter for band C.
  std::vector<double> mBpFltC;

  //! High-pass filter for band D.
  std::vector<double> mHpFltD;

  //! Low-pass filter for based-banded band B.
  std::vector<double> mLpFltB;

  //! Low-pass filter for based-banded band C.
  std::vector<double> mLpFltC;

  //! Low-pass filter for based-banded band D.
  std::vector<double> mLpFltD;

  //! Notch Filter.
  std::vector<double> mNotchFilter;

  //! Whitening Filter.
  std::vector<double> mWhiteningFilter;

  //! Sampling rate band A.
  double mSampleRateBandA;

  //! Decimate factor for band A.
  int mDecimateFactorA;

  //! Sampling rate band B.
  double mSampleRateBandB;

  //! Decimate factor for band B.
  int mDecimateFactorB;

  //! Sampling rate band C.
  double mSampleRateBandC;

  //! Decimate factor for band C.
  int mDecimateFactorC;

  //! Sampling rate band D.
  double mSampleRateBandD;

  //! Decimate factor for band D.
  int mDecimateFactorD;

  //! Time delay due to filter operations.
  double mFilterDelay;

  //! Buffer time shift in sec.
  double mBufferTimeShift;

  //! Buffer for continuous signal processing.
  std::vector<double> buffer;

  //! Output trend band A.
  std::string mPercentageChannelBandA;
  std::string mMaxValueChannelBandA;
  //! Output trend band B.
  std::string mPercentageChannelBandB;
  std::string mMaxValueChannelBandB;
  //! Output trend band C.
  std::string mPercentageChannelBandC;
  std::string mMaxValueChannelBandC;
  //! Output trend band D.
  std::string mPercentageChannelBandD;
  std::string mMaxValueChannelBandD;

  //! Twelve hours plot
  std::list<double> m12bandA;
  std::list<double> m12bandB;
  std::list<double> m12bandC;
  std::list<double> m12bandD;

  //! Gaussian Noise.
  double mMeanA;
  double mStdDevA;
  double mMeanB;
  double mStdDevB;
  double mMeanC;
  double mStdDevC;
  double mMeanD;
  double mStdDevD;

  //! Operation State Condition List
  OperStateCondList *mOSC;
  //! Lock condition name
  std::string mLockConditionName;
  //! Lock condition flag, true if locked.
  bool mLockConditionFlag;

  //! List with warning messages
  std::list<std::string*> mWarningMessage;
  //! List with error messages
  std::list<std::string*> mErrorMessage;
  
  // Private Methods
  //! Estimate Threshold for Noise floor via Monte Carlo simulation with Gaussian white noise.
  void EstimateThreshold();

  //! Prepare data for dmtview plot: down sample.
  void PrepareDataForPlot(std::vector<double>& in,std::vector<float>& ou);
  //! Send empty plot to the DMT-viewer for non locked detector.
  void EmptyPlot();
  //! Send thresholds plot to DMT-viewer.
  void ThresholdPlot();
  //! Send times series to DMT-viewer.
  void TimeSeriesPlot(std::vector<double>& bandA,std::vector<double>& bandB,std::vector<double>& bandC,std::vector<double>& bandD);
  //! Set the Noise floor trend and the twelve hours plot.
  void SetNoiseFloorTrend(std::vector<double>& bandA,std::vector<double>& bandB,std::vector<double>& bandC,std::vector<double>& bandD);
  
  //! Write into html document
  std::string mHTMLStatusFile;
  std::string mHTMLErrorFile;
  void DumpHTMLStatusPage();
  void DumpHTMLErrorPage();
};



#endif     //  __NOISE_FLOOR_MONITOR_H
