#ifndef RAYLEIGHCONFIG_HH
#define RAYLEIGHCONFIG_HH 


#include <stdio.h>


const int max_channame_length = 30;
const int max_file_path_length = 200;
 

/** The RayleighConfig class reads a configuration file containing 
  * input parameters and passes them to the RayleighMonitor class DMT monitor.
  * Adapted from RmsConfig by Ed Daw.
  * @memo Configuration tool for the RayleighMonitor class DMT monitor. 
  * @author Patrick J. Sutton
  * @version 6.1; Last Modified 2002/08/16. 
  */
class RayleighConfig {
 
  public:
 
    /** Construct a RayleighConfig object.
      * @memo Constructor.
      * @param input_param_file_name   Name of configuration file. 
      */
    RayleighConfig( char* input_param_file_name ); 

    /** Destroy a RayleighConfig object.
      * @memo Destructor.  
      */
    ~RayleighConfig();
 
    /** Verify that N, Stride parameters are acceptable.  Returns?
      * @memo Verify that N, Stride parameters are acceptable.  Returns?
      */
    bool RayleighConfig::CheckParameters();  

    /** Length of each data segment (sec). 
      * @memo Length of each data segment (sec).
      */
    double Stride;  

    /** Number of data segments to average. 
      * @memo Number of data segments to average. 
      */
    int N; 

    /** Maximum number of strides to process. 
      * @memo Maximum number of strides to process.      
      */
    int MaxStride;  

    /** Number of channels to process. 
      * @memo Number of channels to process. 
      */
    int number_of_channels;

    /** Pointer to list of channel names. 
      * @memo Pointer to list of channel names. 
      */
    char** channel_names;

    /** Pointer to list of channel rates. 
      * @memo Pointer to list of channel rates. 
      */
    int* channel_rates;

    /** Minimum frequency to analyze.
      * @memo Minimum frequency to analyze. 
      */
    double* f_lower;

    /** Maximum frequency to analyze.
      * @memo Maximum frequency to analyze. 
      */
    double* f_upper;

  private:

};


//====================================== Constructor.  
RayleighConfig::RayleighConfig( char* input_param_file_name )
{
    FILE* in;
    if ( (in=fopen(input_param_file_name,"r")) == NULL ) {
      std::printf("ERROR: cannot open %s.\n",input_param_file_name);
      //exit(0);  // monitor will crash below:
    } else {
      std::fscanf(in,"%lf",&Stride);
      std::fscanf(in,"%d",&N);
      std::fscanf(in,"%d",&MaxStride);
      std::fscanf(in,"%d",&number_of_channels);
      channel_names = new char* [number_of_channels];
      f_lower = new double[number_of_channels];
      f_upper = new double[number_of_channels];
      channel_rates = new int[number_of_channels];
      for(int i=0;i<number_of_channels;++i) {
        channel_names[i] = new char [ max_channame_length ];
        std::fscanf(in,"%s%d%lf%lf",channel_names[i],(channel_rates+i),(f_lower+i),(f_upper+i));
      }
      fclose(in);
    }
}


//====================================== Destructor.  
RayleighConfig::~RayleighConfig()
{
    for(int i=0;i<number_of_channels;++i) {
      delete [] channel_names[i];
    }
    delete [] channel_names;
}


//====================================== Make sure N, Stride are okay.  
bool
RayleighConfig::CheckParameters() 
{
    bool okay = true;
    
    //-------------------- Check that Stride = Integer * 2^{-14}.
    //                     (2^{-14} = 0.006103515625).  :KLUDGE:
    //                     Should check instead that stride 
    //                     is multiple of 1/sampling rate of 
    //                     SLOWEST selected channel.
    //                     [Powers of 2 should be free of roundoff
    //                     error, so don't check for that.]
    int stride_check = int(16384*Stride);
    double decimal_part = 16384*Stride - ((double) stride_check); 
    if (decimal_part != 0.0) {
        std::cerr << "FATAL: Decimal part of Stride must be multiple ";
        std::cerr << "of 1/16384 = 2^{-14}.  EXITING.\n";
        okay = false;
    }

    //-------------------- Check that N > 1.  Can't get variance 
    //                     of a single sample!
    if (N <= 1) {
        std::cerr << "FATAL: Must have N > 1.  EXITING.\n";  
        okay = false;
    }
    
    return okay;
}


#endif     //  RAYLEIGHCONFIG_HH  
