//
//  Class PSD:  Computes power spectrum (with windowing and detrending) 
//  from TSeries data.
//    authors:  Kevin C. Schlaufman (kcs149@psu.edu)
//              Patrick J. Sutton (psutton@ligo.caltech.edu)
//
//  $Id: PSD.hh,v 1.7 2006/08/16 22:46:45 psutton Exp $
//
///////////////////////////////////////////////////////////////////////////


#ifndef PSD_HH
#define PSD_HH


#ifndef __CINT__
#include "Blackman.hh"
#include "FlatTop.hh"
#include "FSpectrum.hh"
#include "Hamming.hh"
#include "Hanning.hh"
#include "Interval.hh"
#include "Time.hh"
#include "TSeries.hh"
#include "Window.hh"
#endif  //------ !def(__CINT__)


/** Power spectrum class.
  * @memo Power spectrum class.
  * @author Kevin C. Schlaufman
  */
class PSD
{
public:
        /** Constructs a PSD object with a Hamming window and the desired 
	  * number of averages.
	  * @memo Constructor with Hamming window.
	  * @param win_choice Pointer to a Hamming window.
	  * @param intervals Number of averages to be used.
	  */
	PSD(Hamming *win_choice, int intervals);

        /** Constructs a PSD object with a Hanning window and the desired 
	  * number of averages.
	  * @memo Constructor with Hanning window.
	  * @param win_choice Pointer to a Hanning window.
	  * @param intervals Number of averages to be used.
	  */
        PSD(Hanning *win_choice, int intervals);

        /** Constructs a PSD object with a FlatTop window and the desired 
	  * number of averages.
	  * @memo Constructor with FlatTop window.
	  * @param win_choice Pointer to a FlatTop window.
	  * @param intervals Number of averages to be used.
	  */
        PSD(FlatTop *win_choice, int intervals);

        /** Constructs a PSD object with a Blackman window and the desired 
	  * number of averages.
	  * @memo Constructor with BlackMan window.
	  * @param win_choice Pointer to a Blackman window.
	  * @param intervals Number of averages to be used.
	  */
        PSD(Blackman *win_choice, int intervals);

        /** Constructs a PSD object with a square (no) window and the desired 
	  * number of averages.
	  * @memo Constructor with square (no) window.
	  * @param win_choice Dummy pointer.
	  * @param intervals Number of averages to be used.
	  */
        PSD(const char *square, int intervals);

        /** Constructs a PSD object with the default window type (Hamming) and 
	  * the desired number of averages.
	  * @memo Constructor with Hamming window.
	  * @param intervals Number of averages to be used.
	  */
	PSD(int intervals);

        /** Constructs a PSD object with the default window type (Hamming) and 
	  * the default number of averages (10).
	  * @memo Constructor with Hamming window and 10 averages.
	  */
	PSD();

	/** Destroy a PSD object.
	  * @memo Destructor.
	  */
	~PSD();

	/** Generates a power spectral density from the given TSeries pointer.
	  * @memo Generates a power spectral density.
	  * @param total FSpectrum in which power spectrum is written.
	  * @param ts Pointer to TSeries holding data to be Fourier transformed.
	  */
	void generate(FSpectrum& total, TSeries* ts);

private:

	int n;
	Hamming *ham;
	Hanning *han;
        FlatTop *flat;
        Blackman *black;
        int choice;
};


inline
PSD::PSD(Hamming *win_choice, int intervals)
{
        n = intervals;
        ham = new Hamming;
        choice = 1;
}

inline
PSD::PSD(Hanning *win_choice, int intervals)
{
        n = intervals;
        han = new Hanning;
        choice = 2;
}

inline
PSD::PSD(FlatTop *win_choice, int intervals)
{
        n = intervals;
        flat = new FlatTop;
        choice = 3;
}

inline
PSD::PSD(Blackman *win_choice, int intervals)
{
        n = intervals;
        black = new Blackman;
        choice = 4;
}

inline
PSD::PSD(const char *square, int intervals)
{
        n = intervals;
        choice = 5;
}

inline
PSD::PSD(int intervals)
{
	n = intervals;
	ham = new Hamming;
        choice = 1;  
}

inline
PSD::PSD()
{
	n = 1;
	ham = new Hamming;       
        choice = 1;  
}

inline
PSD::~PSD()
{
	if(choice == 1)
	    delete ham;
        else if(choice == 2)
	    delete han;
        else if(choice == 3)
	    delete flat;
        else if(choice == 4)
	    delete black;

}

inline
void PSD::generate(FSpectrum& total, TSeries* ts)
{
	// Divide TSeries interval into n sub-intervals of equal length
	Interval seg_length_temp = ts->getInterval();
	seg_length_temp /= ((double)n); 
	const Interval seg_length = seg_length_temp;
	Time t_zero = ts->getStartTime();
	TSeries sub_series;
        
	// Test to determine if seg_length is an integer.  If not, display
	// possible error message
	if ((double)seg_length != (int)((double)seg_length))
	{
		std::cout << "The window function may not work properly if ";
		std::cout << "the window is applied to time segments not an ";
		std::cout << "integer number of seconds long.\n\n";
	}

	// Set window length
	if(choice == 1)
            ham->setWindow((int)((double)seg_length));
        else if(choice == 2)
            han->setWindow((int)((double)seg_length));
        else if(choice == 3)
            flat->setWindow((int)((double)seg_length));
        else if(choice == 4)
            black->setWindow((int)((double)seg_length));

	// Extract data starting with t_zero and lasting for seg_length
	sub_series = ts->extract(t_zero, seg_length);
	
	// Detrend, window sub_series
	sub_series -= sub_series.getAverage();
	if(choice == 1)
            sub_series = ham->apply(sub_series);
        else if(choice == 2)
            sub_series = han->apply(sub_series);
        else if(choice == 3)
            sub_series = flat->apply(sub_series);
        else if(choice == 4)
            sub_series = black->apply(sub_series);

	// Add to running total for power spectrum
	total = FSpectrum(FSeries(sub_series));

	// Increment t_zero
	t_zero += seg_length;

	// Loop to divide TSeries into n sub-series
	if (n > 1) {
		for(int i = 2; i <= n; i++)
		{
			// Extract data starting with t_zero and lasting for 
			// length seg_length
			sub_series = ts->extract(t_zero, seg_length);

			// Detrend, window sub_series
			sub_series -= sub_series.getAverage();
			if(choice == 1)
                            sub_series = ham->apply(sub_series);
                        else if(choice == 2)
                            sub_series = han->apply(sub_series);
                        else if(choice == 3)
                            sub_series = flat->apply(sub_series);
                        else if(choice == 4)
                            sub_series = black->apply(sub_series);

			// Add to running total for power spectrum
			total += FSpectrum(FSeries(sub_series));

			// Increment t_zero
			t_zero += seg_length;
		}
	}

	// Normalize to mean power per data point
	total *= 1.0 / ((double)n);

}

#endif     //  PSD_HH
