/************************************************************
* fastScript.cc                                             *
* plotting filtered time series                             *
*************************************************************
* Author:  Stefan Ballmer, MIT (sballmer@ligo.mit.edu)      *
*          Laura Cadonati, MIT (cadonati@ligo.mit.edu)      *
* Date:    Feb. 27, 2003                                    *
************************************************************/

#include "fastGlitch.hh"

// Bug fix for
// $ROOTSYS/include/TMath.h
//#define isfinite finite

#include "TApplication.h"
#include "TROOT.h"

#include "Dacc.hh"
#include <iostream>
#include <cstdlib>

#include <fstream>
// ROOT header files
#include <TH1.h>
#include <TH2.h>
#include <TLegend.h>
#include <TPad.h>
#include <TCanvas.h>
#include <TStopwatch.h>
#include <TSystem.h>
#include <TStyle.h>
#include <TROOT.h>
#include <TPaveLabel.h>
#include <TText.h>
#include <TObjArray.h>
// EventTool header files
#include <events/Events.hh>
//#include <MyAlgorithm.hh>
// other gds header files
#include <TSeries.hh>
#include <Complex.hh>
#include <Hanning.hh>
#include <Dacc.hh>
#include <DaccIn.hh>
#include <IIRFilter.hh>
#include <FilterBase.hh>
#include <FilterDesign.hh>
#include <Pipe.hh>


#ifdef _USE_FASTAUDIO_
  #include "fastaudio.hh"
#endif

using namespace events;

void help(void);

void TPlotConvert(const TSeries* ts) {

    //----------------------------------  Formatting parameters
    float ymargin = 0.1;  // Fractional margin in y.

    if (!ts) {printf("TPlot: Invalid TSeries pointer.");}

    //----------------------------------  Get the TSeries parameters.
    int nbin = ts->getNSample();
    float stime = ts->getStartTime().totalS();
    if (stime > 1000000.0) stime = 0;
    float step  = ts->getTStep().GetSecs();

    //----------------------------------  Get the data and real length.
    float* DVec = new float[nbin];
    int nwd = ts->getData(nbin,DVec);
    float dtime = stime + step * nwd;

    //----------------------------------  Create and fill the histogram.
    TH1F* hist  = (TH1F*) gROOT->FindObject("TS001");
    if (hist) delete hist;
    hist = new TH1F("TS001", ts->getName(), nwd, stime, dtime);
 
    float ymin = DVec[0];
    float ymax = DVec[0];
    for (int i=0 ; i<nwd ; i++) {
        //-----------------------------  Histogram content in bins 1-N. Bin 0
        //                               has UnderFlow, bin N+1 has OverFlow
        hist->SetBinContent(i+1, DVec[i]);
	if (DVec[i] < ymin) ymin = DVec[i];
	if (DVec[i] > ymax) ymax = DVec[i];
    }
    
    delete[] DVec;
    float yavg = 0.5 * (ymin + ymax);
    float ydif = yavg - ymin;
    ymin = yavg - (1.0 + ymargin)*ydif;
    ymax = yavg + (1.0 + ymargin)*ydif;
    if (ymin < 0.0 && yavg-ydif >= 0.0) ymin = 0.0;
    hist->SetMinimum(ymin);
    hist->SetMaximum(ymax);
}





EXEROOT(fastGlitch)

void fastGlitch::SetOptions(string optstring)
{
  if (optstring  == "-batch" || optstring  == "-b") {
    if(Debug()) cout <<"setting option -batch"<<endl;
    gROOT->SetBatch();
  } else
  if (optstring  == "-nobatch" || optstring  == "-interactive") {
    if(Debug()) cout <<"setting option -nobatch"<<endl;
    gROOT->SetBatch(false);
  } else
  if (optstring  == "-FrameCacheQuery" || optstring  == "-FCQ") {
    if(Debug()) cout <<"setting option -FrameCacheQuery"<<endl;
    useFCQ=true;
  } else
  if (optstring  == "-noFrameCacheQuery" || optstring  == "-noFCQ") {
    if(Debug()) cout <<"setting option -noFrameCacheQuery"<<endl;
    useFCQ=false;
  } else
  if (optstring  == "-print" || optstring  == "-p" || 
      optstring  == "-save"  || optstring  == "-s") {
    if(Debug()) cout <<"setting option -save"<<endl;
    saveplot=true;
  } else
  if (optstring  == "-noprint" || optstring  == "-nosave") {
    if(Debug()) cout <<"setting option -nosave"<<endl;
    saveplot=false;
  } else
  if (optstring  == "-noboxes" || optstring  == "-nolines") {
    if(Debug()) cout <<"setting option -noboxes"<<endl;
    boxtype=0;
  } else
  if (optstring  == "-lines") {
    if(Debug()) cout <<"setting option -lines"<<endl;
    boxtype=1;
  } else
  if (optstring  == "-boxes" ) {
    if(Debug()) cout <<"setting option -boxes"<<endl;
    boxtype=2;
  } 
}

fastGlitch::fastGlitch(int argc, const char *argv[]) 
  : DatEnv(argc, argv)
{
    /****************************************************************
    * PARSE ARGUMENTS                                               *
    ****************************************************************/
    if(Debug()>=9) {
      cout <<"Files already added to Dacc:"<<endl;
      getDacc().listFiles(cout);
    }
    conffile="fastGlitch.conf";
    ttfilename="fastGlitch-TemporaryFile.txt";

    //----------------------------------  Parse the arguments
    // Check whether a new configuration file was specified
    for (int i=1 ; i<argc && argv ; i++) {
        string argi = argv[i];
        if (argi  == "-conf" || argi  == "-config") {
	    if (++i >= argc) return;
	    conffile=argv[i];
	     if(Debug()) cout <<"setting option -conf "<<conffile<<endl;
	}
    }
    // Load the configuration file
    LoadConfFile();
    // Parse the remaining commandline options (override configuration file options)
    for (int i=1 ; i<argc && argv ; i++) { string argi = argv[i]; SetOptions(argi); }
    
    
    if(useFCQ) {
      string frame;
      char command[512];
      unsigned long u1,u2;
      u1=(TFirstSec/16)*16;
      u2=(TFirstSec/16)*16+int(SettleTime+Tbin);     
      sprintf(command,"%s %lu-%lu > %s",FrameCacheQuery.c_str(),u1,u2,ttfilename.c_str());
      if(Debug()) cout <<"  "<<command << endl;
      system(command);
      ifstream ttfile(ttfilename.c_str(),ios::in);
      while(ttfile.good()) {
        ttfile >> frame;
	// Adding the files at this point somehow doesn't work with DatEnv
	// feature disabled for now.
	//if(ttfile.good()) getDacc().addFile(frame.c_str());
      }
      //getDacc().addFile("/data/E9/LLO/L-R-727549920-16.gwf");
      //getDacc().close();
      //getDacc().open();
      ttfile.close();
      if(Debug()) {
        sprintf(command,"cat %s",ttfilename.c_str());
        cout <<"  "<<command << endl;
        system(command);
      }
      sprintf(command,"rm %s",ttfilename.c_str());
      if(Debug()) cout <<"  "<<command << endl;
      system(command);
         
      //getDacc().addFile("/data/E9/LLO/L-R-7275500*");
      if(Debug()>=9) {
        cout <<"Files added to Dacc:"<<endl;
        getDacc().listFiles(cout);
      }
    }
     

    /****************************************************************
    * INPUT PARAMETERS                                              *
    ****************************************************************/
    Nmax = 1000;         // max number of triggers to be overlapped to the time series in Tbin segments

    ts_raw = new TSeries*[Nchan];
    getDacc().setStride(Tbin);


    /************************************************
    * Declare the channels to be analyzed           *
    * NOTE: this is hard-coded for 5 channels only! *
    ************************************************/
  
    for(int ic=0;ic<Nchan;++ic) {
      getDacc().addChannel(ChannelName[ic].c_str()); 
    }
    
    Time tFirst;           // the time we want to start with (used only if TFirstSec>0)
    if (TFirstSec>0) {       /* look for the specified start time */
      tFirst.setS(TFirstSec);  
      getDacc().seek(tFirst);
    }
    
    state=0;  //reinitialize the filters etc
}

fastGlitch::~fastGlitch() 
{
   //delete ChannelName;
   //delete FilterString;
   //delete ts_raw;
}


/************************************************
* ProcessData                                   *
************************************************/
void
fastGlitch::ProcessData(void) {


if(state==0) {
  /*****************************************
  * design the filters                     *
  *****************************************/
    fsample      = new double[Nchan];
    filter       = new Pipe*[Nchan];
    filterdesign = new FilterDesign*[Nchan];
    for(int ic=0;ic<Nchan;++ic) {
      ts_raw[ic] = getDacc().refData(ChannelName[ic].c_str());
      fsample[ic]=1.0/ts_raw[ic]->getTStep();
      filterdesign[ic] = new FilterDesign(fsample[ic]);
      if( !filterdesign[ic]->filter(FilterString[ic].c_str()) ) {
         cerr << "invalid filter. " << FilterString[ic] <<" using butter('HighPass',4,30.0)"<< endl;
	 FilterString[ic]="butter('HighPass',4,30.0)";
	 filterdesign[ic]->filter(FilterString[ic].c_str());
      }
      filter[ic]=filterdesign[ic]->release();
    }
  /*****************************************
  * Filter the data                        *
  *****************************************/
  TSeries ts[Nchan];     // will be windowed time series
  for(int ic=0;ic<Nchan;++ic) {
    ts[ic]=filter[ic]->apply(*ts_raw[ic]);
    ts[ic].setName((ChannelName[ic]+" "+FilterString[ic]).c_str());
  }
  state=1;
} else
if(state==1) {
  // Fix potential initial time mismatch
  TFirstSec=getDacc().getFillTime().getS()-int(SettleTime);

  /*****************************************
  * Filter the data                        *
  *****************************************/
  TSeries ts[Nchan];     // will be windowed time series
  for(int ic=0;ic<Nchan;++ic) {
    ts[ic]=filter[ic]->apply(*ts_raw[ic]);
    ts[ic].setName((ChannelName[ic]+" "+FilterString[ic]).c_str());
  }


//==========================================================================//
// START HARD-CODED DEFINITIONS          

    /*****************************************************
    *  Point here to the frame folder and the xml files  *
    *****************************************************/
 
    events::Set s1;
    if(OvelayXMLFile!="" && boxtype > 0) s1.Restore(OvelayXMLFile.c_str());
   
    events::Set sStride; 
    events::Set sStride2; 

    if(Debug()) cout << s1.Size() << " triggers in this interval"<< endl; 


// STOP HARD-CODED DEFINITIONS          
//==========================================================================//

    /************************************
    * Declarations                      *
    ************************************/
    Hanning Wn(8192);      // Hanning window

    //Float_t TempDat;
    TH1F** hTSeries = new TH1F*[Nchan]; 
    //TH2F** hSpec = new TH2F*[Nchan];
    string* HistoName = new string [Nchan]; 
    string* HistoTitle = new string [Nchan]; 
    string* NameTSeries = new string [Nchan]; 
    TLegend** aLegend = new TLegend*[Nchan]; 
    TPad** aPad = new TPad*[Nchan]; 

    for (int ic=0; ic<Nchan; ++ic){ 
      char buf[30];
      HistoTitle[ic]=ChannelName[ic]; /* spectrogram title */
      sprintf(buf,"Spe%i",ic);
      HistoName[ic]=buf;              /* spectrogram name */
      sprintf(buf,"TSeries%i",ic);
      NameTSeries[ic]=buf;            /* power spectra plot name */
    } 

    /*****************************************
    * Graphics settings                      *
    *****************************************/
    gStyle->SetPalette(1);
    gStyle->SetFrameFillColor(19);
    gStyle->SetCanvasColor(39);  
    gStyle->SetOptStat(0);
    gStyle->SetOptTitle(1);


  if(Debug()) cout << getDacc().getFile() << endl;
/***********************************************************************************************************************************************************************/
  char t0[128];
  Time Time0=ts_raw[0]->getStartTime();
  if( Time0.getS()!=TFirstSec+int(SettleTime) ) {
    cerr << "fastGlitch: internal time error!"<< endl;
  }
  TimeStr(Time0,t0,"%M %d %Y,  %H:%N:%S %Z");
  sprintf(t0,"%s = %lu sec %lu nanosec",t0,Time0.getS(),Time0.getN());
  
  sStride.Select(s1,events::Column("time")<ts->getEndTime() && events::Column("time")>=ts->getStartTime()); 
  //sStride.Select(events::Column("central_freq")<3000. && events::Column("central_freq")>=400.); 
  //sStride.Select(events::Column("central_freq")>=400.); 

  if(Debug()) cout << sStride.Size() << " TFClusters triggers in this interval"<< endl; 


  /*********************************************
  * Convert time series into ROOT histograms   *
  *********************************************/

  for(int ic=0; ic<Nchan; ic++) {   // loop over the channels 
    TPlotConvert(&(ts[ic])); 
    TH1F* hist  = (TH1F*) gROOT->FindObject("TS001");
    if (hist) {
      if (gROOT->FindObject(NameTSeries[ic].c_str())) delete gROOT->FindObject(NameTSeries[ic].c_str());
      hTSeries[ic]=(TH1F*)hist->Clone(NameTSeries[ic].c_str()); 
      int iCol=ic%8+2; 
      if (iCol==5) iCol=1;
      hTSeries[ic]->SetLineColor(iCol);
    }
  }
  
  //================================
  // PLOT HISTOGRAMS ON A 1x4 CANVAS
  //================================

  gStyle->SetOptTitle(0);
 
  if (gROOT->FindObject("TSeriesCanv")) delete gROOT->FindObject("TSeriesCanv");
  TCanvas* TSeriesCanv=new TCanvas("TSeriesCanv","TSeries",400,10,1000,1000);
  TSeriesCanv->cd();

  TPaveLabel* title = new TPaveLabel(0.1,0.96,0.9,0.99, t0);
  title->SetFillColor(17);
  title->SetTextFont(52);
  title->Draw();

  Double_t ylow,yup;
  for (int ic=0; ic<Nchan; ic++){
    char padName[30];
    sprintf(padName,"pad%i",ic);
    if (gROOT->FindObject(padName)) delete gROOT->FindObject(padName);
    yup =0.01*ic/Nchan + 0.95*(Nchan-ic)/Nchan;
    ylow=0.01*(ic+1)/Nchan + 0.95*(Nchan-ic-1)/Nchan;
    aPad[ic] = new TPad(padName,"The pad with the histogram", 0.01,ylow,0.99,yup,17);
  }

  for (int ic=0; ic<Nchan; ic++){
    aPad[ic]->Draw();  
    aPad[ic]->SetGrid();
    aPad[ic]->SetBottomMargin(0.15);
    aPad[ic]->SetTopMargin(0.01);
    aPad[ic]->SetLeftMargin(0.1);
    aPad[ic]->SetRightMargin(0.01);
  }
  
  for (int ic=0; ic<Nchan; ic++){
    aPad[ic]->cd(); 
    (hTSeries[ic])->GetXaxis()->SetLabelSize(0.08);
    (hTSeries[ic])->GetYaxis()->SetLabelSize(0.08);
    (hTSeries[ic])->Draw();
    aLegend[ic] = new TLegend(0.7,0.85,0.975,0.95); 
    aLegend[ic]->AddEntry(hTSeries[ic],hTSeries[ic]->GetTitle(),"l");
    aLegend[ic]->Draw();
  }

  TText* l2 = new TText(0.50, 0.05, "Time (seconds)");
  l2->SetNDC(true);
  l2->SetTextAlign(22);
  l2->SetTextSize(0.07);
  l2->SetTextAngle(0.);
  l2->Draw("");

  if(boxtype>0) {
    Int_t nn=sStride.Size();
      TObjArray TrigBox(nn);
      double TrigAmpl[Nmax];
      double TrigFreq[Nmax];
      double TrigStart[Nmax];
      double TrigStop[Nmax];
      double TrigPow[Nmax];
      Value val;
      double aTrigStart; 
      double aTrigDur;
      double aTrigFreq;
      double aTrigAmpl;
      double aTrigPow;
      int i=0;
    if (nn > 0) {
      for (Iterator itr=sStride.Begin(); itr != sStride.End(); ++itr){
        Time TSta=(*itr).GetTime();
        Interval TSta1=TSta-Time0;      // this is an Interval
        aTrigStart = TSta1;    // this is a double
        (*itr).GetValue("duration",val);
        val.Write(aTrigDur);
        (*itr).GetValue("central_freq",val);
        val.Write(aTrigFreq);
        (*itr).GetValue("amplitude",val);
        val.Write(aTrigAmpl);
        (*itr).GetValue("snr",val);
        val.Write(aTrigPow);
        TrigStart[i]=aTrigStart;
        TrigStop[i]=aTrigStart+aTrigDur;
        TrigAmpl[i]=aTrigAmpl;
        TrigPow[i]=aTrigPow*aTrigAmpl;
        TrigFreq[i]=aTrigFreq;
        i++;
      }
    }

  // hTSeries[1]->SetMaximum(4.);
  //   hTSeries[1]->SetMinimum(-4.);

    for(int ic =0; ic<Nchan; ic++){
      aPad[ic]->cd();
      Float_t Ymin=hTSeries[ic]->GetMinimum();
      Float_t Ymax=hTSeries[ic]->GetMaximum();
      for (int ii=0; ii<nn; ii++){
        if(boxtype==1) TrigPow[ii]=0;
        TBox* b=new TBox(TrigStart[ii],Ymin+(Ymax-Ymin)/10,TrigStop[ii],abs(TrigPow[ii])*10*(Ymax-Ymin)/10000+Ymin+(Ymax-Ymin)/10);
        b->SetFillColor(33);
        b->SetFillStyle(0);
        b->SetLineColor(1); 
        if (ic==3 || ic ==2) b->SetLineColor(2);
        b->SetLineWidth(2);
        b->SetLineStyle(1);
        TrigBox[ii]=b;
        i++;     
      }
      if (nn > 0) TrigBox.Draw();
    }
  }
    
  gSystem->ProcessEvents();

  //================================
  // Save the plot
  //================================
  if(saveplot) {
    string convert="convert -density 144x144";
    string plotname;
    char buf[512];
    char command[1024];
    const char* gf=GraphicsFormat.c_str();
    sprintf(buf,"%lu.%u.eps",Time0.getS(),int(Tbin));
    plotname=GraphicsPrefix+buf;
    if(Debug()) cout << "Saving graphics as eps"<<endl;
    TSeriesCanv->Print(plotname.c_str(),"eps");
    if(strstr(gf,"pdf")) {
       sprintf(command,"epstopdf %s",plotname.c_str());
       if(Debug()) cout << "Converting eps to pdf:\n  "<<command<<endl;
       system(command);
    }
    if(strstr(gf,"png")) {
       sprintf(command,"%s %s %s%lu.%u.png",convert.c_str(),plotname.c_str(),GraphicsPrefix.c_str(),Time0.getS(),int(Tbin));
       if(Debug()) cout << "Converting eps to png:\n  "<<command<<endl;
       system(command);
    }
    if(strstr(gf,"gif")) {
       sprintf(command,"%s %s %s%lu.%u.gif",convert.c_str(),plotname.c_str(),GraphicsPrefix.c_str(),Time0.getS(),int(Tbin));
       if(Debug()) cout << "Converting eps to gif:\n  "<<command<<endl;
       system(command);
    }
    if(strstr(gf,"jpg")) {
       sprintf(command,"%s %s %s%lu.%u.jpg",convert.c_str(),plotname.c_str(),GraphicsPrefix.c_str(),Time0.getS(),int(Tbin));
       if(Debug()) cout << "Converting eps to jpg:\n  "<<command<<endl;
       system(command);
    }
    if(!strstr(gf,"eps")) {
       sprintf(command,"rm -f %s",plotname.c_str());
       if(Debug()) cout << "removing eps file:\n  "<<command<<endl;
       system(command);
    }
  }
  
  if(Debug()) cout <<"fastGlitch Time: "<<TFirstSec+int(SettleTime)<<endl;
  if(Debug()) cout <<"DatEnv     Time: "<<getDacc().getFillTime()<<endl;

#ifdef _USE_FASTAUDIO_
  if(playaudiochannel >= 0 && playaudiochannel < Nchan) {
    if(Debug()) cout <<"Playing channel "<<playaudiochannel<<" on sound card.."<<endl;  
    playTSeries(ts[playaudiochannel,Debug()]);
  }
#endif

  //================================
  // Display the graphics and wait
  // if not in batchmode
  //================================
  char k=0;
  if(gROOT->IsBatch())
  {
    if(timeFromXML) {
      SetNextState();
    } else finish();
  } else {
    for(;k!='n';)
    {
     cout << "Close the time series plot to continue..."<<endl;
     for(;gROOT->FindObject("TSeriesCanv");) gSystem->ProcessEvents();
      if(timeFromXML) { cout << "Press: q for exit, n for next "<<Tbin<<" sec, t for next trigger."<<endl;
      } else { cout << "Press: q for exit, n for next "<<Tbin<<" sec."<<endl; } 
      k=getchar();
      if(k=='n') { TFirstSec+=int(Tbin); }
      if(k=='t') { if(timeFromXML) SetNextState(); k='n'; }
      if(k=='q') { finish(); k='n'; }
    }
  }


  //================================
  // Cleanup
  //================================
  //delete fsample;
  //delete filter;
  //delete filterdesign;
  //finish();
  }
}

void fastGlitch::SetNextState(void)
{
  ++trigTimes_itr;
  if(trigTimes_itr!=trigTimes.End()) {
    unsigned long endtime,newtime;
    endtime=TFirstSec+int(SettleTime+Tbin);
    newtime=(trigTimes_itr->GetTime()).getS();
    if(Debug()) cout << "New trigger time is: "<<newtime<<endl;
    // make sure the next trigger was not already on the plot
    while(trigTimes_itr!=trigTimes.End() && endtime > newtime) {
      ++trigTimes_itr;
        if(trigTimes_itr!=trigTimes.End()) {
	  newtime=(trigTimes_itr->GetTime()).getS();
	  if(Debug()) cout << "New trigger time is: "<<newtime<<endl;
	}
    }
    if(trigTimes_itr!=trigTimes.End()) {
      // we already know that endtime <= newtime
      if(newtime<=endtime+int(SettleTime)) {
	// just display the next Tbin seconds (now new filter settling)
	TFirstSec+=int(Tbin);
	state=1;
      } else {
	// newtime > endtime + SettleTime
	// seek the new start time, new filter settling needed
	TFirstSec=newtime-int(SettleTime+TimeOffset);
	if(TFirstSec < endtime) TFirstSec=endtime;
	if(Debug()) cout << "seeking "<<TFirstSec<<endl;
	getDacc().seek(Time(TFirstSec,0));
	state=0;
      }
    } else finish();	
  } else finish();
}

// Utility that removes all characters c from in 
char* strremchr(char* in,char c) {
  char *p;
  char *cp=in;
  for(;*cp && *cp!=c;cp++);
  for(p=cp;*cp;cp++){
    if(*cp!=c) *(p++)=*cp;
  }
  *p=0;
  return in;
}

void fastGlitch::LoadConfFile(void)
{
    /**********************************************
    *  Read the config file                       *
    **********************************************/
   string Tagstring,Configstring;
   char buf[8192];
   char* p;

   // set the default values
   Nchan=0;
   Tbin = 10.0;       // time interval for each segment (sec)   SettleTime=Tbin;   // has to be Tbin
   SettleTime=Tbin;
   TimeOffset=0;
   TFirstSec=0;      // if not 0, this is used as start time for the scan
   timeFromXML=false;
   OvelayXMLFile="";
   GraphicsPrefix="";
   GraphicsFormat="";
   FrameCacheQuery="FrameCacheQuery llo L R";
   saveplot=false;
   boxtype=1;
   useFCQ=false;
   playaudiochannel=-1;

   
   // open configuration file
   if(Debug()) cout <<"\nReading configuration file "<<conffile.c_str()<<endl;
   ifstream file(conffile.c_str(),ios::in);

   while(file.good()) {
      //read and prepare the input
      file.getline(buf,8192);
      if(*buf && *buf!='#' && *buf!='%') {
        if((p=strchr(buf,'='))){
          *p=0;
	  strremchr(buf,' ');
	  Tagstring=buf;
          ++p;
          while(*p==' ') {++p;}
          Configstring=p;
	  Tagstring=buf;
          if(Debug()) cout << Tagstring <<" = "<<Configstring<<endl;
	
 	  // parse the conffile
	       if(Tagstring == "Time"    )       { ParseTime(Configstring);}
	  else if(Tagstring == "TimeOffset")     { TimeOffset=atof(Configstring.c_str()); }
	  else if(Tagstring == "Channels")       { ReadChannels(Configstring,file); }
	  else if(Tagstring == "OverlayTriggers"){ OvelayXMLFile=Configstring; }
	  else if(Tagstring == "Stride")         { SettleTime=Tbin=atof(Configstring.c_str()); }
	  else if(Tagstring == "GraphicsPrefix") { GraphicsPrefix=Configstring; }
	  else if(Tagstring == "GraphicsFormat") { GraphicsFormat=Configstring; }
	  else if(Tagstring == "FrameCacheQuery") { FrameCacheQuery=Configstring; }
	  else if(Tagstring == "PlayAudioChannel") { playaudiochannel=atoi(Configstring.c_str()); }
	  else if(Debug()) cout << "  --> Tagstring "<<Tagstring<<" not recognized"<<endl;
        } else if((p=strchr(buf,'-'))){
	  Tagstring=p;
          SetOptions(Tagstring);
        }
      }
   }

   // close configuration file
   file.close();
   
   // subtract SettleTime from TFirstSec
   if (TFirstSec>0) {
     TFirstSec-=int(SettleTime + TimeOffset);
     if(Debug()) cout << "Subtracting SettleTime ("<<SettleTime<<") and TimeOffset ("<<TimeOffset<<"), TFirstSec= "<< TFirstSec<< endl;
   } else if(Debug()) cout << "TFirstSec= "<< TFirstSec<< endl;
   
   if(Debug()) cout <<"finished reading configuration file "<<conffile.c_str()<<endl<<endl;
}

 // ParseTime sets TFirstSec
 void fastGlitch::ParseTime(string timestring)
{
   string command;
   unsigned long timeInt;

   if( strstr(timestring.c_str(),".xml") || strstr(timestring.c_str(),".XML"))
   {
      trigTimesXML=timestring;
      trigTimes.Restore(trigTimesXML.c_str());
      trigTimes.Sort();
      if(Debug()>=2) {
        cout <<"trigger times reported in "<<trigTimesXML<<":" << endl;
        trigTimes.DumpColumn("TIME");
      }
      if(trigTimes.Size()>0) {
        trigTimes_itr=trigTimes.Begin();
	if(trigTimes_itr!=trigTimes.End()) {
          timeInt=(trigTimes_itr->GetTime()).getS();
          timeFromXML=true;
        } else { timeInt=0; timeFromXML=false; }
      } else { timeInt=0; timeFromXML=false; }
   }
   else
   if( strchr(timestring.c_str(),'.')   || strchr(timestring.c_str(),':')   || strchr(timestring.c_str(),'T') || 
       strstr(timestring.c_str(),"now") || strstr(timestring.c_str(),"NOW") || strstr(timestring.c_str(),"Now") )
   {
      if(Debug()) cout << "need time conversion:"<<endl;
      command=string("tconvert ") + timestring + string(" > ") + ttfilename;
      if(Debug()) cout <<"  "<<command << endl;
      system(command.c_str());
      ifstream ttfile(ttfilename.c_str(),ios::in);
      ttfile >> timeInt;
      ttfile.close();
      command=string("rm ")+ttfilename;
      if(Debug()) cout <<"  "<<command << endl;
      system(command.c_str());
      timeFromXML=false;
   }
   else { timeInt=atoi(timestring.c_str()); timeFromXML=false; }
   
   if(Debug()) cout << timestring << " = "<< timeInt << endl;
   TFirstSec=timeInt;
}

// ReadChannels sets Nchan, ChannelName[ic] and FilterString[ic]
void fastGlitch::ReadChannels(string Nchanstring,ifstream &file)
{
   // Read the number of channels
   Nchan=atoi(Nchanstring.c_str());       // number of channels to be analyzed
   
   // Read the channels and filters
   ChannelName  = new string[Nchan];
   FilterString = new string[Nchan];
   for(int ic=0;ic<Nchan;++ic) {
      file >> ChannelName[ic] >> FilterString[ic];
      if(Debug()) cout << ChannelName[ic] <<" "<< FilterString[ic]<< endl;
   }
}
