#ifndef TRIGTABLE_HH
#define TRIGTABLE_HH

#include "xsil/MetaTable.hh"
#include <string>

namespace trig {

  static const int nProcessKeyBytes = 13;

  class Segment;
  class TrigBase;
  class TrigProc;

  namespace DBTypes {
    typedef float       real_4;
    typedef double      real_8;
    typedef int         integer_4;
    typedef std::string varchar;
    typedef std::string character;
  }

  /**  The TrigTable class is a meta-database interface producing an xsil 
    *  meta database table.
    */
  class TrigTable : public xsil::MetaTable {
  public:
    TrigTable(bool alphaPID=false);
    ~TrigTable(void);
    void addRow(const TrigBase& t);
    void check(const char* title) const;
  private:
    DBTypes::integer_4 start_time;
    DBTypes::integer_4 start_time_ns;
    DBTypes::integer_4 priority;
    DBTypes::integer_4 disposition;
    DBTypes::varchar   name;
    DBTypes::varchar   subtype;
    DBTypes::character ifo;
    DBTypes::character event_id;
    DBTypes::real_4    duration;
    DBTypes::real_4    size;
    DBTypes::real_4    significance;
    DBTypes::real_4    frequency;
    DBTypes::real_4    bandwid;
    DBTypes::real_4    peak_time;
    DBTypes::real_4    avg_time;
    DBTypes::real_4    time_sigma;
    DBTypes::real_4    freq_peak;
    DBTypes::real_4    freq_avg;
    DBTypes::real_4    freq_sigma;
    DBTypes::real_4    noise_power;
    DBTypes::real_4    signal_power;
    DBTypes::integer_4 pixel_count;
    DBTypes::real_4    confidence;
    
    //----------------------------------  UCVecs and their data vectors
    DBTypes::integer_4 binarydata_length;
    unsigned char data[1024];
    xsil::UCVec binarydata;
    
    int proc_id_length;
    unsigned char processID[nProcessKeyBytes];
    xsil::UCVec process_id;

    //----------------------------------  Alternative process ID
    bool altPID;
    DBTypes::character PIDstr;
  };

  /**  The ProcTable class is a meta-database interface producing an xsil 
    *  meta database table.
    */
  class ProcTable : public xsil::MetaTable {
  public:
    ProcTable(bool alphaPID=false);
    ~ProcTable(void);
    void addRow(const TrigProc& s);
    void check(const char* title) const;
  private:
    void setColumns(void);
  private:
    DBTypes::character program;
    DBTypes::varchar   version;
    DBTypes::varchar   cvsname;
    DBTypes::integer_4 cvstime;
    DBTypes::varchar   comment;
    DBTypes::varchar   node;
    DBTypes::character username;
    DBTypes::integer_4 unix_procid;
    DBTypes::integer_4 start_time;
    DBTypes::integer_4 end_time;
    DBTypes::character ifo;
    
    //----------------------------------  UCVec and its data vectors.
    int proc_id_length;
    unsigned char processID[nProcessKeyBytes];
    xsil::UCVec process_id;

    //----------------------------------  Alternative process ID
    bool altPID;
    DBTypes::character PIDstr;
  };

  /**  The SegTable class is a meta-database interface producing an xsil 
    *  meta database table.
    */
  class SegTable : public xsil::MetaTable {
  public:
    SegTable(bool alphaPID=false);
    ~SegTable(void);
    void addRow(const Segment& t);
    void check(const char* title) const;
  private:
    DBTypes::varchar   group;
    DBTypes::integer_4 version;
    DBTypes::integer_4 start_time;
    DBTypes::integer_4 start_time_ns;
    DBTypes::integer_4 end_time;
    DBTypes::integer_4 end_time_ns;
    
    //----------------------------------  UCVec and its data vectors.
    int proc_id_length;
    unsigned char processID[nProcessKeyBytes];
    xsil::UCVec process_id;

    //----------------------------------  Alternative process ID
    bool altPID;
    DBTypes::character PIDstr;
  };

  //=====================================================================
  //
  //   The mySql database needs a few tables
  //
  //=====================================================================

  class DQSegDef {
  public:
    DQSegDef(void);
    DQSegDef(const Segment& seg);
    bool operator==(const DQSegDef& x) const;
    int getVersion(void) const;
    const std::string& refProcess(void) const;
    const std::string& refDefinerID(void) const;
    const std::string& refIfos(void) const;
    const std::string& refName(void) const;
    const std::string& refComment(void) const;
    void setDefinerID(const std::string& defid);
  private:
    std::string mProcID;
    std::string mDefID;
    std::string mIfos;
    std::string mName;
    int         mVersion;
    std::string mComment;
  };

  /**  The SegTable class is a meta-database interface producing a segment
    *  database table.
    */
  class DQSegTable : public xsil::MetaTable {
  public:
    DQSegTable(void);
    ~DQSegTable(void);
    void addRow(const Segment& s);
    void check(const char* title) const;
  private:
    DBTypes::integer_4 start_time;
    DBTypes::integer_4 end_time;
    DBTypes::integer_4 active;
    DBTypes::varchar   segid;
    DBTypes::varchar   procid;
  };

  /**  The DQSegDefTable class is a meta-database interface producing a segment
    *  definition database table.
    */
  class DQSegDefTable : public xsil::MetaTable {
  public:
    DQSegDefTable(void);
    ~DQSegDefTable(void);
    void addRow(const DQSegDef& df);
    void check(const char* title) const;
  private:
    DBTypes::varchar   defid;
    DBTypes::varchar   procid;
    DBTypes::varchar   run;
    DBTypes::varchar   ifos;
    DBTypes::varchar   name;
    DBTypes::integer_4 version;
    DBTypes::varchar   comment;
  };

  /**  The SegMapTable class is a meta-database interface producing a
    *  segment map table.
    */
  class DQSegMapTable : public xsil::MetaTable {
  public:
    DQSegMapTable(void);
    ~DQSegMapTable(void);
    void addRow(const std::string& segid, const std::string& defid, 
		const std::string& procid);
    void addRow(const Segment& s, const DQSegDef& d);
    void check(const char* title) const;
  private:
    DBTypes::varchar segid;
    DBTypes::varchar defid;
    DBTypes::varchar procid;
  };
}

#endif // TRIGTABLE_HH

