//
//    Implementation of Data accessor channel class.
//
#if defined(sun) && !defined(__EXTENSIONS__)
#define __EXTENSIONS__
#endif

#include "Channel.hh"
#include "DVecType.hh"
#include "framecpp/FrameH.hh"
#include "framecpp/FrRawData.hh"
#include "framecpp/FrVect.hh"
#include "FrVectRef.hh"
#include "general/gpstime.hh"
#include "TSeries.hh"
#include "FSeries.hh"
#include <iostream>

using FrameCPP::FrAdcData;
using FrameCPP::FrProcData;
using FrameCPP::FrRawData;
using FrameCPP::FrSimData;
using FrameCPP::FrVect;

using namespace std;

//======================================  Translate from General::GPSTime
inline Time
GetGPSTime(const General::GPSTime& T) {
    return Time(T.GetSeconds(), T.GetNanoseconds());
}

//======================================  Default constructor
Channel::Channel(void) 
  : mDebug(false), mSample(0.0), mLast(0), mNSample(0), mAccVal(0.0),
    mUserPtr(0), mFUserPtr(0), myPtr(0), mType(kUnknown)
{
}

//======================================  Define a channel
Channel::Channel(const char* name, TSeries** ctlv, uint_type decim, ChanType t) 
  : mDebug(false), mSample(0.0), mLast(0), mNSample(0), mAccVal(0.0),
    mFUserPtr(0), myPtr(0), mType(t)
{
    //----------------------------------  Copy the name string.
    mName = name;

    //----------------------------------  Set the decimation factor;
    mDecim8 = decim;
    if (decim == 0) mDecim8 = 1;

    //----------------------------------  Set up the user pointer.
    mUserPtr = ctlv;
    if (!mUserPtr) mUserPtr = reinterpret_cast<TSeries**>(&myPtr);
}

//======================================  Define a channel
Channel::Channel(const char* name, FSeries** ctlv) 
  : mName(name), mDecim8(0), mDebug(false), mSample(0.0), mLast(0), 
    mNSample(0), mAccVal(0.0), mUserPtr(0), myPtr(0), mType(kFSeries)
{
    //----------------------------------  Set up the user pointer.
    mFUserPtr = ctlv;
    if (!mFUserPtr) mFUserPtr = reinterpret_cast<FSeries**>(&myPtr);
}

//======================================  Define a channel
Channel::Channel(const Channel& d) 
  : mUserPtr(0), mFUserPtr(0), myPtr(0)
{
    *this = d;
}

//======================================  Define a channel
const Channel&
Channel::operator=(const Channel& d) 
{
    mDebug   = d.mDebug; 
    mSample  = d.mSample; 
    mLast    = d.mLast;
    mNSample = d.mNSample;
    mAccVal  = d.mAccVal;
    deleteSeries();
    mType    = d.mType;
    mName    = d.mName;
    mDecim8  = d.mDecim8;
    if (d.mUserPtr != reinterpret_cast<TSeries* const*>(&d.myPtr)) {
        mUserPtr = d.mUserPtr;
    } else {
        if (d.myPtr) myPtr = new TSeries(*reinterpret_cast<const TSeries*>(d.myPtr));
        mUserPtr = reinterpret_cast<TSeries**>(&myPtr);
    }
    if (d.mFUserPtr != reinterpret_cast<FSeries* const*>(&d.myPtr)) {
        mFUserPtr = d.mFUserPtr;
    } else {
        if (d.myPtr) myPtr = new FSeries(*reinterpret_cast<const FSeries*>(d.myPtr));
        mFUserPtr = reinterpret_cast<FSeries**>(&myPtr);
    }
    return *this;
}

//======================================  Destroy a channel entry.
Channel::~Channel() {
    deleteSeries();
}

//======================================  Delete locally manageed series
void 
Channel::deleteSeries(void) {
    if (mUserPtr  == reinterpret_cast<TSeries**>(&myPtr) && myPtr) 
        delete reinterpret_cast<TSeries*>(myPtr);
    if (mFUserPtr == reinterpret_cast<FSeries**>(&myPtr) && myPtr) 
        delete reinterpret_cast<FSeries*>(myPtr);
    myPtr = 0;
}

//======================================  Allocate a Time series.
void 
Channel::allocate(Interval Stride) {
    if (mType == kFSeries ) {
	FSeries* uPtr = *mFUserPtr;
	if (uPtr) {
	    uPtr->clear();

	//------------------------------  DVec type unknown for direct copies
	} else {
	    uPtr = new FSeries();
	    uPtr->setName(mName.c_str());
	    *mFUserPtr = uPtr;
	}
    } else {

	//------------------------------  Calculate the number of words
        size_type nw = 0;
	if (double(mSample) > 0 && double(Stride) > 0) {
	    nw = size_type(Stride/mSample + mDecim8 - 0.5) / mDecim8;
	}

	//------------------------------  Clear and Resize TSeries if it exists
	TSeries* uPtr = *mUserPtr;
	if (uPtr) {
	    uPtr->Clear();
	    reserve(Stride);

	//------------------------------  DVec type unknown for direct copies
	} else if (mDecim8 == 1) {
	    uPtr = new TSeries(mLast, mSample);
	    uPtr->setName(mName.c_str());

	//------------------------------  float DVec for decimated streams
	} else {
	    uPtr = new TSeries(mLast, mSample, nw);
	    uPtr->setName(mName.c_str());
	}
	*mUserPtr = uPtr;
    }

    mLast    = Time(0);
    mAccVal  = 0.0;
}

//======================================  Reserve space in the target series.
void 
Channel::reserve(Interval dT) {
    if (mType != kFSeries && double(mSample) > 0 && double(dT) > 0) {
        size_type nw = size_type(dT/mSample + mDecim8 - 0.5) / mDecim8;
	if (*mUserPtr) {
	    DVector* dv = (*mUserPtr)->refDVect();
	    if (dv) dv->reserve(nw);
	}
    }
}

//======================================  Fill frame data into the series.
//
//    Return codes from FillSeries
//        0  Successful completion
//	 -1  Frame start not contiguous to previous data
//	 -2  Sample rate isn't compatible with previous data.
//	 -3  Requested data not found in current frame
//	 -5  Frame data are not self-consistent.
//	 -6  TSeries is not allocated.
//	 -7  Unsupported data type
//
int
Channel::FillSeries(const FrameCPP::FrameH* frame, Interval off, Interval dT)
{
    Time t0(GetGPSTime(frame->GetGTime()));

    const FrRawData*  raw  = frame->GetRawData();
    const FrAdcData*  adc  = 0;
    const FrProcData* proc = 0;
    const FrSimData*  sim  = 0;

    //----------------------------------  Get the adc data.
    switch (mType) {
    case kRaw:
	if (!raw) {
	    if (mDebug) cout << "No raw data in frame." << endl;
	} else {
	    adc = *(raw->RefFirstAdc().find(mName, raw->RefFirstAdc().begin()));
	    if (adc) return FillSeries(adc, t0, off, dT);
	    if (mDebug) cout << "ADC " << mName << " not in frame." << endl;
	}
	break;

    //----------------------------------  Get the processed data.
    case kProcessed:
    case kFSeries:
	proc = *(frame->RefProcData().find(mName, frame->RefProcData().begin()));
	if (proc) return FillSeries(proc, t0, off, dT);
	if (mDebug) cout << "ProcData " << mName << " not in frame." << endl;
	break;

    //----------------------------------  Get the simulated data.
    case kSimulated:
	sim = *(frame->RefSimData().find(mName, frame->RefSimData().begin()));
	if (proc) return FillSeries(sim, t0, off, dT);
	if (mDebug) cout << "SimData " << mName << " not in frame." << endl;
	break;

    //----------------------------------  Look everywhere
    case kUnknown:
	if (raw) {
	    adc = *(raw->RefFirstAdc().find(mName, raw->RefFirstAdc().begin()));
	    if (adc) return FillSeries(adc, t0, off, dT);
	}
	proc = *(frame->RefProcData().find(mName, frame->RefProcData().begin()));
	if (proc) return FillSeries(proc, t0, off, dT);
	if (mDebug) cout << "Channel " << mName << " not in frame." << endl;
    }
    return -3;
}

//======================================  Fill adc data into the series.
//
//    Return codes from FillSeries
//        0  Successful completion
//	 -1  Frame start not contiguous to previous data
//	 -2  Sample rate isn't compatible with previous data.
//	 -3  Requested data not found in current frame
//	 -5  Frame data are not self-consistent.
//	 -6  TSeries is not allocated.
//	 -7  Unsupported data type
//
int
Channel::FillSeries(const FrAdcData* adc, const Time& t0, 
		    Interval off, Interval dT)
{
    //----------------------------------  Get data  pointers.
    const FrVect* vect = adc->RefData().front();
    
    int rc = setSample(Interval(1./adc->GetSampleRate()));
    if (rc) return rc;

    int fix = 0;
    //----------------------------------  Fix S2 word order screwup
    //
    //  From Jan 6, 2003 through July 29, 2003 pairs of 16-bit words 
    //  were swapped in the raw data frames in channels read out by 
    //  the H1 suspension and pem ADCUs. These channels have channel
    //  IDs in the range 7805 - 7956 (inclusive) for the S2 runs. 
    //  The following code sets fix=1 if the data was taken during 
    //  S2 and the channel number is in the appropriate range.
    //
    if (t0 < Time(735000000) && t0 >= Time(729000000)) {
        int ichan = adc->GetChannelNumber();
	if (ichan >= 7805 && ichan < 7957) fix = 1;
    }

    //----------------------------------  Fill the series from the FrVect
    rc = FillSeries(vect, t0, off, dT, fix);
    if (!rc) {
        mType = kRaw;
	(*mUserPtr)->combineStatus(adc->GetDataValid() ? 1 : 0);
	(*mUserPtr)->setF0(adc->GetFShift());
    }
    return rc;
}

//======================================  Fill processed data into series.
//
//    Return codes from FillSeries
//        0  Successful completion
//	 -1  Frame start not contiguous to previous data
//	 -2  Sample rate isn't compatible with previous data.
//	 -3  Requested data not found in current frame
//	 -5  Frame data are not self-consistent.
//	 -6  TSeries is not allocated.
//	 -7  Unsupported data type
//
int
Channel::FillSeries(const FrProcData* proc, const Time& t0, 
		    Interval off, Interval dT)
{
    FrVectRef vect = proc->RefData().front();
    Interval  pdoff = proc->GetTimeOffset();

    //-----------------------------------  Unpack a t-series channel.
    int rc(0);
    if (proc->GetType() == FrProcData::TIME_SERIES && mType != kFSeries) {
	rc = setSample(vect);
	if (!rc) rc = FillSeries(vect, t0+pdoff, off-pdoff, dT, 0);
	if (!rc) mType = kProcessed;
    } else if (proc->GetType() == FrProcData::FREQUENCY_SERIES && 
	       mType == kFSeries) {
	rc = FillFSeries(vect, t0+pdoff, proc->GetTRange());
	if (!rc) mLast = t0 + off + dT;
    } else {
        if (getDebug()) cout << "FrProcData channel " << getName()
			     << " wrong series type." << endl;
	rc = -7;
    }

    return rc;
}

//======================================  Fill sim data into the series.
//
//    Return codes from FillSeries
//        0  Successful completion
//	 -1  Frame start not contiguous to previous data
//	 -2  Sample rate isn't compatible with previous data.
//	 -3  Requested data not found in current frame
//	 -5  Frame data are not self-consistent.
//	 -6  TSeries is not allocated.
//	 -7  Unsupported data type
//
int
Channel::FillSeries(const FrSimData* sim, const Time& t0, 
		    Interval off, Interval dT)
{
    FrVectRef vect = sim->RefData().front();
    int rc = setSample(vect);
    if (rc) return rc;
    rc = FillSeries( vect, t0, off, dT, 0);
    if (!rc) mType = kSimulated;
    return rc;
}

//====================================== Set sample from the FrVec dimension
int
Channel::setSample(const FrVectRef& vect) {
    if (vect.empty()) return -3;
    return setSample(Interval(vect.getDimDx(0)));
}

//====================================== Set the sample interval
int 
Channel::setSample(Interval dT) {
    if (dT <= Interval(0.0)) {
        if (getDebug()) cout << "Channel " << mName 
			     << " sample rate not valid." << endl;
	return -5;
    } else if (!mSample) {
        mSample = dT;
    } else if (mSample != dT) {
        if (mDebug) cout << "Channel " << mName << " sample rate changed from "
			 << mSample << " to " << dT << endl;
        return -2;
    }
    return 0;
}

//======================================  Fill FrVect into the series.
//
//    Return codes from FillSeries
//        0  Successful completion
//	 -1  Frame start not contiguous to previous data
//	 -2  Sample rate isn't compatible with previous data.
//	 -3  Requested data not found in current frame
//	 -5  Frame data are not self-consistent.
//	 -6  TSeries is not allocated.
//	 -7  Unsupported data type
//
int
Channel::FillSeries(const FrVectRef& vect, const Time& t0, 
		    Interval off, Interval dT, int fix)
{
    if (fix == 1) {
        cerr << "Requested fix not implemented" << endl;
	throw std::runtime_error("Fix not implemented");
    }

    //-----------------------------------  Check arguments.
    if (!*mUserPtr) {
        if (getDebug()) cout << "TSeries is not allocated." << endl;
	return -6;
    }
    if (vect.empty()) return -3;


    //----------------------------------  Check the start time.
    size_type   first = vect.getIndex(0, off);
    size_type    last = vect.getIndex(0, off+dT);
    Time       tStart = t0 + double(first)*mSample;
    Interval reSample = mSample;

    //----------------------------------  Get raw FrVect data
    DVector* dv = vect.getDVector(first, last);
    int      rc = dv ? 0: -6;

    //----------------------------------  Perform optional decimation
    if (mDecim8 != 1 && !rc) {
	DVecType<double> dvd(*dv);
	delete[] dv;
	dv = 0;

	tStart  -= mSample * double(mNSample);
        reSample = mSample * double(mDecim8);

	size_type inx=0;
	size_type len=dvd.getLength();
        for (size_type i=0; i<len; ++i) {
	    mAccVal += dvd[i];
	    if (++mNSample >= mDecim8) {
	        dvd[inx++] = mAccVal / double(mNSample);
		mAccVal  = 0;
		mNSample = 0;
	    }
	}

	if (inx) dv = dvd.Extract(inx);
    }

    //----------------------------------  Append the (resampled) data vector
    if (!rc) {
        TSeries ts(tStart, reSample, dv);
	rc = (*mUserPtr)->Append(ts);
    }

    //----------------------------------  Clean up and go away.
    if (!rc) {
        mLast = t0 + double(last)*mSample;
    } else if (mDebug) {
        cout << "FillSeries: Error from TSeries, rc = " << rc << endl;
	cout << "   mLast=" << mLast << " mSample= " << mSample << endl;
	(*mUserPtr)->Dump(cout);
    }

    return rc;
}

//======================================  Fill FSeries from an FrVect.
//
//    Return codes from FillSeries
//        0  Successful completion
//	 -1  Frame start not contiguous to previous data
//	 -2  Sample rate isn't compatible with previous data.
//	 -3  Requested data not found in current frame
//	 -5  Frame data are not self-consistent.
//	 -6  Series is not allocated.
//	 -7  Unsupported data type
//
int
Channel::FillFSeries(const FrVectRef& vect, const Time& t0, Interval dT) {
    //-----------------------------------  Check arguments.
    int rc=0;
    if (!mFUserPtr || !*mFUserPtr) {
	rc = -6;
    } else if (vect.empty()) {
        rc = -3;


    //-----------------------------------  Fill FSeries
    } else {
        double f0 = vect.getDimX0(0);
	double dF = vect.getDimDx(0);
	DVector* dv = vect.getDVector();
	**mFUserPtr=FSeries(f0, dF, t0, dT, dv);
	if (!dv) rc = -7;
    }

    //-----------------------------------  Error handling
    if (rc && mDebug) {
        cout << "FillFSeries: Error from FSeries, rc = " << rc << endl;
	cout << "   mLast=" << mLast << " mSample= " << mSample << endl;
	(*mFUserPtr)->Dump(cout);
    }
    return rc;
}
