//
//    Multiple streams of data access implementation
//
#include "MultiDacc.hh"
#include <stdio.h>
#include <iostream>

using namespace std;

//  dTcontig is the maximum acceptable discontinuity in nsec.
static const int dTcontig = 1;

//--------------------------------------  Default (Null) constructor.
MultiDacc::MultiDacc(void)
{
}

//--------------------------------------  Construct and connect to source.
MultiDacc::MultiDacc(const char* Source, const Time& STime)
{
  addMulti(Source);
}

//--------------------------------------  Destructor.
MultiDacc::~MultiDacc(){
  close();
  mChanList.clear();
  for(int i=0;i<(int)mIn.size(); i++) {
    delete mIn[i];
  }
}

void
MultiDacc::close() {
  for(int i=0;i<(int)mIn.size(); i++) {
    mIn[i]->close();
  }
}

int
MultiDacc::open() {
  for(int i=0;i<(int)mIn.size(); i++) {
    int j=mIn[i]->open();
    if(j!=0) return j;
  }
  return 0;
}

void
MultiDacc::setBuffer(int N) {
  for(int i=0;i<(int)mIn.size(); i++) {
    mIn[i]->setBuffer(N);
  }
}

void
MultiDacc::setDebug(int N) {
  for(int i=0;i<(int)mIn.size(); i++) {
    mIn[i]->setDebug(N);
  }
}

void
MultiDacc::setIgnoreMissingChannel(bool yn) {
  for(int i=0;i<(int)mIn.size(); i++) {
    mIn[i]->setIgnoreMissingChannel(yn);
  }
}

void
MultiDacc::setNoWait(bool now) {
  for(int i=0;i<(int)mIn.size(); i++) {
    mIn[i]->setNoWait(now);
  }
}

void
MultiDacc::setStride(Interval Dt) {
  for(int i=0;i<(int)mIn.size(); i++) {
    mIn[i]->setStride(Dt);
  }
}

int
MultiDacc::getDebug() {
  int i=(int)mIn.size();
  if(i>0)
    return mIn[0]->getDebug();
  else
    return 0;
}

Time
MultiDacc::getTotalFrames() const {
  int i=(int)mIn.size();
  if(i>0)
    return mIn[0]->getTotalFrames();
  else
    return (Time)0;
}

Time
MultiDacc::getCurrentTime() const {
  int i=(int)mIn.size();
  if(i>0)
    return mIn[0]->getCurrentTime();
  else
    return (Time)0;
}

Interval
MultiDacc::getStride() const {
  int i=(int)mIn.size();
  if(i>0)
    return mIn[0]->getStride();
  else
    return 0.0;
}



//--------------------------------------  Add a channel to the request list.
void 
MultiDacc::addChannel(const char* Name, int id, int Decimate, TSeries **TSptr) {
  chan_iter i=findChannel(Name);
  if (i != mChanList.end()) {
    if (getDebug()) cout << "Existing channel " << Name 
			 << " entry replaced." << endl;
    mChanList.erase(i);
  }
  ChannelIndex newchan(Name, id);
  mChanList.push_back(newchan);
  int index=getDaccIndex(Name);
  if(index!=-1) {
    mIn[index]->addChannel(Name, Decimate, TSptr);
  }
  else {
    if (getDebug()) cout << "Channel " << Name 
			 << " does not exist in data streams" << endl;
    chan_iter j=findChannel(Name);
    if (j != mChanList.end()) {
      mChanList.erase(j);
    }
  }
}

//--------------------------------------  Add a channel to the request list.
void 
MultiDacc::addFSeries(const char* Name, int id, FSeries **FSptr) {
  chan_iter i=findChannel(Name);
  if (i != mChanList.end()) {
    if (getDebug()) cout << "Existing channel " << Name 
			 << " entry replaced." << endl;
    mChanList.erase(i);
  }
  ChannelIndex newchan(Name, id);
  mChanList.push_back(newchan);
  int index=getDaccIndex(Name);
  if(index!=-1) {
    mIn[index]->addFSeries(Name, FSptr);
  }
  else {
    if (getDebug()) cout << "Channel " << Name 
			 << " does not exist in data streams" << endl;
    chan_iter j=findChannel(Name);
    if (j != mChanList.end()) {
      mChanList.erase(j);
    }
  }
}

//--------------------------------------  Add a channel to the request list.
void 
MultiDacc::addRaw(const char* Name, int id, int Decimate, TSeries **TSptr) {
  chan_iter i=findChannel(Name);
  if (i != mChanList.end()) {
    if (getDebug()) cout << "Existing channel " << Name 
			 << " entry replaced." << endl;
    mChanList.erase(i);
  }
  ChannelIndex newchan(Name, id);
  mChanList.push_back(newchan);
  int index=getDaccIndex(Name);
  if(index!=-1) {
    mIn[index]->addRaw(Name, Decimate, TSptr);
  }
  else {
    if (getDebug()) cout << "Channel " << Name 
			 << " does not exist in data streams" << endl;
    chan_iter j=findChannel(Name);
    if (j != mChanList.end()) {
      mChanList.erase(j);
    }
  }
}

//--------------------------------------  Add a channel to the request list.
void 
MultiDacc::addProcessed(const char* Name, int id, int Decimate, TSeries **TSptr) {
  chan_iter i=findChannel(Name);
  if (i != mChanList.end()) {
    if (getDebug()) cout << "Existing channel " << Name 
			 << " entry replaced." << endl;
    mChanList.erase(i);
  }
  ChannelIndex newchan(Name, id);
  mChanList.push_back(newchan);
  int index=getDaccIndex(Name);
  if(index!=-1) {
    mIn[index]->addProcessed(Name, Decimate, TSptr);
  }
  else {
    if (getDebug()) cout << "Channel " << Name 
			 << " does not exist in data streams" << endl;
    chan_iter j=findChannel(Name);
    if (j != mChanList.end()) {
      mChanList.erase(j);
    }
  }
}

//--------------------------------------  Add a channel to the request list.
void 
MultiDacc::addSimulated(const char* Name, int id, int Decimate, TSeries **TSptr) {
  chan_iter i=findChannel(Name);
  if (i != mChanList.end()) {
    if (getDebug()) cout << "Existing channel " << Name 
			 << " entry replaced." << endl;
    mChanList.erase(i);
  }
  ChannelIndex newchan(Name, id);
  mChanList.push_back(newchan);
  int index=getDaccIndex(Name);
  if(index!=-1) {
    mIn[index]->addSimulated(Name, Decimate, TSptr);
  }
  else {
    if (getDebug()) cout << "Channel " << Name 
			 << " does not exist in data streams" << endl;
    chan_iter j=findChannel(Name);
    if (j != mChanList.end()) {
      mChanList.erase(j);
    }
  }
}

//--------------------------------------  See if channel was requested.
bool 
MultiDacc::isChannelRead(const char* Name) const {
  return findChannel(Name) != mChanList.end();
}

//--------------------------------------  List requested channels
ostream&
MultiDacc::list(ostream& out) const {
  char buf[2048];
  sprintf (buf, "Channel\n");
  out << buf;
  for (const_chan_iter i=mChanList.begin() ; i != mChanList.end() ; i++) {
    sprintf (buf, "%-25s \n", i->getName());
    out << buf;
  }
  return out;
}


//--------------------------------------  Find channel in request list.
MultiDacc::const_chan_iter 
MultiDacc::findChannel(const char* Name) const {
  for (const_chan_iter i=mChanList.begin() ; i != mChanList.end() ; i++) {
    if (i->EqName(Name)) return i;
  }
  return mChanList.end();
}

MultiDacc::chan_iter 
MultiDacc::findChannel(const char* Name) {
  for (chan_iter i=mChanList.begin() ; i != mChanList.end() ; i++) {
    if (i->EqName(Name)) return i;
  }
  return mChanList.end();
}

//--------------------------------------  Remove channel from request list.
void 
MultiDacc::rmChannel(const char* Name) {
  int index=getDaccIndex(Name);
  if(index!=-1) {
    mIn[index]->rmChannel(Name);
  }
  chan_iter i=findChannel(Name);
  if (i != mChanList.end()) {
    mChanList.erase(i);
  }
}

//--------------------------------------  Read a valid frame if necessary
int 
MultiDacc::synch(void) {
  for(int i=0;i<(int)mIn.size(); i++) {
    int j=mIn[i]->synch();
    if(j!=0)
      return j;
  }
  return 0;
}

//--------------------------------------  Fill requested series containers.
//    Error codes:
//      0  Successful completion
//     -1  Frame start not contiguous to previous data
//     -2  Sample rate isn't compatible with previous data.
//     -3  Requested data not found in current frame 
//     -4  Error reading frame
//     -5  Frame data are not self-consistent.
//     -6  TSeries is not allocated.
//     -7  Unsupported data type
//     -8  Interrupter by signal.
int 
MultiDacc::fillData(Interval Stride, bool start) {
  int rc = 0;
  for (int i=0;i<(int)mIn.size(); i++) {
    rc = mIn[i]->fillData(Stride,start);
  }
  return rc;
}

//--------------------------------------  Fill data from a single frame
void
MultiDacc::zeroChans(Interval dT) {
  for (int i=0;i<(int)mIn.size(); i++) {
    mIn[i]->zeroChans(dT);
  }
}

//--------------------------------------  Fill data from a single frame
int
MultiDacc::fillChans(Interval Offset, Interval dT) {
  int rc = 0;
  for (int i=0;i<(int)mIn.size(); i++) {
    rc = mIn[i]->fillChans(Offset, dT);
  }
  return rc;
}

int
MultiDacc::getDaccIndex(const char* name) {
  chan_iter i=findChannel(name); 
  if (i != mChanList.end()) {
    int index=i->getIndex();
    if(index==-1) {
      // find a way to detect where is the channel automatically
      // not so easy, may do it later
      /*
      int j;
      for(j=0;j<(int)mIn.size();j++) {
	cout<<"entering loop"<<endl;
	const FrameCPP::FrAdcData* abc;
	cout<<"before find"<<endl;
	abc=mIn[j]->nextAdc(name);
	cout<<"after find"<<endl;
	if(abc) {
	  cout<<"before set"<<endl;
	  i->setIndex(j);
	  cout<<"after set"<<endl;
	  return j;
	}
	const FrameCPP::FrProcData* proc;
	proc = mIn[j]->nextProc(name);
	if(proc) {
	  i->setIndex(j);
	  return j;
	}
	else {
	  const FrameCPP::FrSimData* sim;
	  sim = mIn[j]->nextSim(name);
	  if(sim) {
	    i->setIndex(j);
	    return j;
	  }
	}
      }
      if(j==(int)mIn.size())
	return -1;
      */
   }
    else
      return index;
  }
  return -1;
}

//--------------------------------------  Flush the specified data
int 
MultiDacc::flush(Interval Stride) {
  int rc = 0;
  for (int i=0;i<(int)mIn.size(); i++) {
    rc = mIn[i]->flush(Stride);
  }
  return rc;
}

//--------------------------------------  Open a file for input.
int
MultiDacc::seek(Time STime) {
  int rc = 0;
  for (int i=0;i<(int)mIn.size(); i++) {
    rc = mIn[i]->seek(STime);
  }
  return rc;
}


//--------------------------------------  Get a reference to the named Channel

TSeries* 
MultiDacc::refData(const char *name) {
  chan_iter i=findChannel(name);
  if (i != mChanList.end()) {
    int index=getDaccIndex(name);
    if(index!=-1)
      return mIn[index]->refData(name);
  }
  return (TSeries*) 0;
}

FSeries* 
MultiDacc::refFData(const char *name) {
  chan_iter i=findChannel(name);
  if (i != mChanList.end()) {
    int index=getDaccIndex(name);
    if(index!=-1)
      return mIn[index]->refFData(name);
  }
  return (FSeries*) 0;
}
