#ifndef MULTIDACC_HH
#define MULTIDACC_HH

#include "Dacc.hh"
#include "Time.hh"
#include "Interval.hh"
#include "ChannelIndex.hh"
#include "MultiList.hh"
#include <string>
#include <iosfwd>

using namespace std;

class ChanList;   // Hide the channel list from root.
class TSeries;
class FSeries;

/**  MultiDacc methods are used to access multiple data channels from multiple
 *  frame file lists. MultiDacc has the same interfaces with Dacc. MultiDacc
 *  is basically a vector of Dacc's. 
 *  @memo LIGO Multiple stream data access class.
 *  @author  Junwei Cao
 *  @version 0.1; Last modified Jan 15, 2005
 */
class MultiDacc
{
public:
  //--------------------------------------  Constructors and destructors.
  /**  Default (Null) constructor.
   */
  MultiDacc(void);
  
  /**  Construct and connect to source (a list of frame file lists).
   */
  explicit MultiDacc(const char* Source, const Time& start=Time(0));
  
  
  /**  Destructor.
   */
  ~MultiDacc(void);

  void close(void);

  int open(void);
  
  //---------------------------------------  Status manipulation
  /**  Set the DaccIn buffer count.
   *  @memo Set the DaccIn Buffer count.
   */
  void setBuffer(int N);
  
  /**  If the debug flag is set, explanatory messages will be printed 
   *  when an error is encountered.
   *  @memo Set the debug flag.
   */
  void setDebug(int N);

  /**  If the debug flag is set, explanatory messages will be printed 
   *  when an error is encountered.
   *  @memo Set the debug flag.
   */
  void setIgnoreMissingChannel(bool yn);

  /**  Set the noWait flag as specified. If noWait is set, the Dacc 
   *  methods that would otherwise wait for online data (e.g. synch, 
   *  fillData) will instead return with error code -8.
   *  @memo Set the noWait flag.
   */
  void setNoWait(bool now=true);

  /**  The default stride is used if a time interval is not specified
   *  when fillData() is invoked.
   *  @memo Define the default Stride.
   */
  void setStride(Interval Dt);

  //--------------------------------------  Channel list manipulation
  /**  The specified channel is added to the list of channels of
   *  the corresponding Dacc.
   *  
   */
  void addChannel(const char* Name, int id=-1, int Decimate=0, TSeries **TSptr=0);
  
  /**  Add an process frequency series to the list. The purpose of the 
   *  method is similar to that of #addChannel# except that the 
   *  channel being searched for is a processed data frequency series.
   *  @memo Add an FSeries data channel to the request list.
   *  @param Name  Name of channel to collect data from.
   *  @param FSptr Address of an FSeries pointer.
   */
  void addFSeries(const char* Name, int id=-1, FSeries **FSptr=0);
  
  /**  Add a Processed data channel to the list. The purpose of the method 
   *  and its arguments are identical to those for #addChannel# except 
   *  that any search for the channel is limited to Processed data.
   *  @memo Add a processed data channel to the request list.
   *  @param Name     Name of channel to collect data from.
   *  @param Decimate Decimation factor.
   *  @param TSptr    Address of TSeries pointer.
   */
  void addProcessed(const char* Name, int id=-1, int Decimate=0, TSeries **TSptr=0);

  /**  Add an Adc channel to the list. The purpose of the method and its
   *  arguments are identical to those for #addChannel# except that any 
   *  search for the channel is limited to the raw ADC data.
   *  @memo Add a raw data channel to the request list.
   *  @param Name     Name of channel to collect data from.
   *  @param Decimate Decimation factor.
   *  @param TSptr    Address of TSeries pointer.
   */
  void addRaw(const char* Name, int id=-1, int Decimate=0, TSeries **TSptr=0);
  
  /**  Add a Simulated data channel to the list. The purpose of the method 
   *  and its arguments are identical to those for #addChannel# except 
   *  that any search for the channel is limited to Simulated data.
   *  @memo Add a simulated data channel to the request list.
   *  @param Name     Name of channel to collect data from.
   *  @param Decimate Decimation factor.
   *  @param TSptr    Address of TSeries pointer.
   */
  void addSimulated(const char* Name, int id=-1, int Decimate=0, TSeries **TSptr=0);
  
  /**  The named channel is removed from the request list.
   *  @memo Remove a channel from the request list.
   */
  void rmChannel(const char* Name);
  
  /**  The data pointer for the frame currently in memory is incremented
   *  past the specified time. If the interval is not specified, 
   *  negative or zero, the rest of the data in the current frame are 
   *  skipped. If a discontinuity in the frame times is found, #flush#
   *  stops skipping at the start of the new data segment.
   *  @return Non-zero if an end of file is found.
   *  @memo Skip data.
   *  @param Stride Time interval to be skipped.
   */
  int flush(Interval Stride=Interval(0.0));
  
  int getDebug(void);

  Time getTotalFrames(void) const;

  Time getCurrentTime(void) const;

  Interval getStride(void) const;

  /**  Data from channels specified by addChannel() are copied to TSeries
   *  objects.
   *  @memo   Fill the requested TSeries.
   *  @return 
   *  \begin{tabular}{|r|l|} \\
   0 & Successful completion \\
   -1 & Frame start not contiguous to previous data \\
   -2 & Sample rate isn't compatible with previous data. \\
   -3 & Requested data not found in current frame \\
   -4 & Error reading frame \\
   -5 & Frame data are not self-consistent. \\
   -6 & TSeries is not allocated. \\
   -7 & Unsupported data type \\
   -8 & Signal received while reading \\
   *  \end{tabular}
   *  @param Stride Time interval to be read in.
   *  @param start  Start a stride.
   */
  int fillData(Interval Stride=Interval(0.0), bool start=true);

  /**  Zero all TSeries in the channel list. Allocate sufficient
   *  memory to hold the given sample time.
   *  @memo   Reset channel TSeries.
   *  @param  Dt     Time interval to be used in allocating memory.
   */
  void zeroChans(Interval Dt);

  /**  Copy data to channel TSeries from a single frame.
   *  @memo   Fill the requested TSeries from a single frame.
   *  @return 
   *  \begin{tabular}{|r|l|} \\
   0 & Successful completion \\
   -1 & Frame start not contiguous to previous data \\
   -2 & Sample rate isn't compatible with previous data. \\
   -3 & Requested data not found in current frame \\
   -4 & Error reading frame \\
   -5 & Frame data are not self-consistent. \\
   -6 & TSeries is not allocated. \\
   -7 & Unsupported data type \\
   *  \end{tabular}
   *  @param  Offset Offset of start of data from start of fram.
   *  @param  Dt     Time interval to be read in.
   */
  int fillChans(Interval Offset, Interval Dt);
  
  /**  Frames are read from the specified input file(s) until one 
   *  containing the specified time is found.
   *  @memo Skip to the specified time.
   *  @return Zero if open was successful.
   */
  int seek(Time STime=Time(0, 0));

  /**  The frame that will provide the data for the next read operation 
   *  is read into memory.
   *  @memo Synchronize the current frame buffer.
   *  @return Zero if successful or -1 if unable to read in a frame.
   */
  int synch(void);
    
  /**  Test to see is the names channel has been requested..
   *  @memo Test if channel was requested.
   *  @return True if the specified channel has been requested.
   *  @param Name Channel name to be tested.
   */
  bool isChannelRead(const char* Name) const;
  
  /**  A formatted list of requested channels, their decimation factors, 
   *  and latest time copied is printed to the specified output stream. 
   *  @memo List requested channels.
   */
  std::ostream& list(std::ostream& out) const;
  
  void addMulti(const char* name);
  void addSingle(const char* name);
  unsigned int getNDacc();
  Dacc* getDacc(unsigned int i);
  /**  Returns a pointer to the last TSeries filled for the named channel.
   *  @memo Get a pointer to the data.
   */
  TSeries* refData(const char* name);

  /**  Returns a pointer to the last TSeries filled for the named channel.
   *  @memo Get a pointer to the data.
   */
  FSeries* refFData(const char* name);
  
private:
#ifndef __CINT__
  typedef std::list<ChannelIndex> ChannelList;
  typedef ChannelList::iterator chan_iter;
  typedef ChannelList::const_iterator const_chan_iter;
  
  const_chan_iter findChannel(const char* Name) const;
  chan_iter       findChannel(const char* Name);
  /**  List of requested channels.
   *  @memo List of requested channels.
   */
  ChannelList mChanList;

  int getDaccIndex(const char* Name);
#endif
  MultiList mList;

  ///  Frame Input object
  typedef std::vector<Dacc*> DaccVector;
  DaccVector mIn;
};

//======================================  Inline methods
#ifndef __CINT__
inline void
MultiDacc::addMulti(const char* name) {
  mList.addMulti(name);
  for(unsigned int i=0;i<mList.size(); i++) {
    Dacc* newDacc = new Dacc();
    newDacc->addFileList(mList.getList(i).c_str());
    mIn.push_back(newDacc);
  }
}
inline void
MultiDacc::addSingle(const char* name) {
  mList.addSingle(name);
  for(unsigned int i=0;i<mList.size(); i++) {
    Dacc* newDacc = new Dacc();
    newDacc->addFileList(mList.getList(i).c_str());
    mIn.push_back(newDacc);
  }
}
inline unsigned int
MultiDacc::getNDacc() {
  return mList.size();
}
inline Dacc*
MultiDacc::getDacc(unsigned int i) {
  return mIn[i];
}
#endif // !defined(__CINT__)

#endif // MULTIDACC_HH
