//
//  Class FDFilter: General frequency-domain filter class.
//    version:  0.3 
//       date:  2004.09.24   
//    authors:  Patrick J. Sutton (psutton@ligo.caltech.edu)
//              John G. Zweizig (jzweizig@ligo.caltech.edu)
//
//  Validation done:
//  Cursory: Can call all methods with reasonable output.
//
///////////////////////////////////////////////////////////////////////////


#include "FDFilter.hh"

#ifndef __CINT__
#include "DVector.hh"
#include <iostream>
#include <stdexcept>
#endif  //------ !def(__CINT__)

using namespace std;

//======================================  FDFilter constructor
FDFilter::FDFilter(void)
{
    //cout << "FDFilter::FDFilter called.\n";
    isFSeries = false;
    return;
}

//======================================  FDFilter data constructor
FDFilter::FDFilter(const FSpectrum& transfunc)
{
    //cout << "FDFilter::FDFilter called.\n";
    FSpecFilter = transfunc;
    isFSeries = false;
    return;
}

//======================================  FDFilter data constructor
FDFilter::FDFilter(const FSeries& transfunc)
{
    FSeriesFilter = transfunc;
    isFSeries = true;
    // cout << "FDFilter::FDFilter called.\n";
    return;
}

//======================================  FDFilter destructor
FDFilter::~FDFilter(void)
{
    //cout << "FDFilter::~FDFilter called.\n";
    return;
}

//======================================  Clone a filter
FDFilter*
FDFilter::clone(void) const {
    return new FDFilter(*this);
}

//======================================  Apply the filter to an FSpectrum
void 
FDFilter::Apply(const FSpectrum& Int_PSD, FSpectrum& out_data) {
    // cout << "FDFilter::Apply called.\n";

	//---------- Useful quantities

	//---------- Check that frequency resolution of interpolated 
	//           calibration data matches that of Fspectrum to be
	//           calibrated.
	double freq_step = FSpecFilter.getFStep();
	if (freq_step != Int_PSD.getFStep()) {
		throw runtime_error("FDFilter WARNING:  Frequency resolution of data and filter do not match.");
	}
	
	//---------- Determine frequency range for which we have both 
	//           filter and data.
	double freq_max = FSpecFilter.getHighFreq();
	double freq_min = FSpecFilter.getLowFreq();
	if (freq_min < Int_PSD.getLowFreq()) {
		freq_min = Int_PSD.getLowFreq();
	}
	if (freq_max > Int_PSD.getHighFreq()) {
		freq_max = Int_PSD.getHighFreq();
	}

	//---------- Extract appropriate subsection of data, filter.
	if (freq_min != Int_PSD.getLowFreq() || 
	    freq_max != Int_PSD.getHighFreq()  ) {
	    out_data = Int_PSD.extract(freq_min, freq_max-freq_min+freq_step);
	} else if (&out_data != &Int_PSD) {
	    out_data = Int_PSD;
	}

	unsigned long jMin = FSpecFilter.getBin(freq_min);
	unsigned long jMax = FSpecFilter.getBin(freq_max+freq_step);
	if (jMin == 0 && jMax == FSpecFilter.getNStep()) {
	    *(out_data.refDVect()) *= *FSpecFilter.refDVect();
	} else {
	    DVector* temp = FSpecFilter.refDVect()->Extract(jMin, jMax-jMin);
	    *(out_data.refDVect()) *= *temp;
	    delete temp;
	}
}

//======================================  Apply the filter to an FSeries
void 
FDFilter::Apply(const FSeries& Int_PSD, FSeries& out_data) {

	//---------- Check that frequency resolution of interpolated 
	//           calibration data matches that of Fspectrum to be
	//           calibrated.
	double freq_step = FSeriesFilter.getFStep();
	if (freq_step != Int_PSD.getFStep()) {
		throw runtime_error("FDFilter WARNING:  Frequency resolution of data and filter do not match.");
	}
	
	//---------- Determine frequency range for which we have both 
	//           filter and data.
	double freq_max = FSpecFilter.getHighFreq();
	double freq_min = FSpecFilter.getLowFreq();
	if (freq_min < Int_PSD.getLowFreq()) {
		freq_min = Int_PSD.getLowFreq();
	}
	if (freq_max > Int_PSD.getHighFreq()) {
		freq_max = Int_PSD.getHighFreq();
	}

	//---------- Extract appropriate subsection of data, filter.
	if (freq_min != Int_PSD.getLowFreq() || 
	    freq_max != Int_PSD.getHighFreq()  ) {
	    out_data = Int_PSD.extract(freq_min, freq_max-freq_min+freq_step);
	} else if (&out_data != &Int_PSD) {
	    out_data = Int_PSD;
	}

	unsigned long jMin = FSeriesFilter.getBin(freq_min);
	unsigned long jMax = FSeriesFilter.getBin(freq_max+freq_step);
	out_data.refDVect()->mpy(0, *FSeriesFilter.refDVect(), jMin, jMax-jMin);
}

//======================================  Apply the filter to an FSpectrum
FSpectrum 
FDFilter::Apply(const FSpectrum& Int_PSD) {
    FSpectrum out;
    Apply(Int_PSD, out);
    return out;
}

//======================================  Apply the filter to an FSeries
FSeries 
FDFilter::Apply(const FSeries& Int_DFT) {
    FSeries out;
    Apply(Int_DFT, out);
    return out;
}

//======================================  Set the filter spectrum.
void 
FDFilter::setFDFilter(const FSpectrum& newfilter)
{
    // cout << "FDFilter::setFDFilter called.\n";
    FSpecFilter = newfilter;
}
