#ifndef TABLE_COLUMN_HH
#define TABLE_COLUMN_HH

#include "cell.hh"
#include <vector>

//======================================  Column class definition
namespace table {

  /**  Column class.
    *  @author John G. Zweizig
    *  @memo Column class.
    *  @version 1.2; Last modified: May 2, 2006
    */
  class Column {
  public:
    typedef unsigned int row_index;
  public:
    /**  Construct an empty column with the specified title.
      *  @memo construct a column.
      */
    Column(const std::string& title);

    /**  Add a row to the end of a column and fll the new cell with the 
      *  specified value.
      *  @memo add a row
      *  @param c Cell initialization value.
      */
    void addRow(const Cell& c);

    /**  Get a character pointer to the title string.
      *  @memo Get the Title.
      *  @return Constant pointer to the title string.
      */
    const char* getTitle(void) const;

    /**  Get the type of the column.
      */
    Cell::cell_type getType(void) const;

    /**  Get the maximum width of the column strings.
      */
    unsigned int getWidth(void) const;

    /**  Get the external data type
      */
    const char* getXType(void) const;

    /**  Insert a new row at the specified position and initialize the 
      *  new cell with the specified cell value.
      *  @param b4 Row number before which data are inserted.
      *  @param c  New cell content.
      */
    void insertRow(row_index b4, const Cell& c);

    /**  Insert multiple rows before the specified row. The inserted data 
      *  are initialized with the contents of the #n# rows of column #c#
      *  starting with row #s#.
      */
    void insertRow(row_index b4, const Column& c, row_index s, row_index n);
    bool less(row_index i, row_index j) const;
    bool lesn(row_index i, row_index j) const;

    /**  Move a row from one index to another, leaving all other rows in the
      *  same position relative to one another. The 'from' row is takes
      *  the place of the 'to' row and all intermediate rows are shifted 
      *  one position to fill the vacated 'from' position.
      *  @memo Move a row.
      *  @param from index of the row to be moved.
      *  @param to   index of the row to be replaced by the from row.
      */
    void move(row_index from, row_index to);

    /**  Return a reference to the specified cell.
      */
    const Cell& operator[](row_index i) const;
    Cell& operator[](row_index i);
    
    const Cell& refCell(row_index i) const;
    Cell& refCell(row_index i);

    /**  Return a constant reference to the column title string.
      */
    const std::string& refTitle(void) const;

    /**  Select the specified rows. All rows that are not selected are 
      *  deleted.
      */
    void selectRows(const std::vector<bool>& mask);

    /**  Set the column format.
      */
    void setFormat(const std::string& title);    

    /**  Set the column justification.
      */
    void setJustify(const std::string& title);    

    /**  Set the column title.
      */
    void setTitle(const std::string& title);    

    /**  Set the External data type.
      */
    void setXType(const std::string& title);    

    /**  Sum the numeric contents of the column. #Sum()# will throw an 
      *  exception if one or more cells are non-numeric.
      */
    double Sum(void) const throw(std::runtime_error);

    /**  Get the number of rows in this column.
     */
    row_index size(void) const;

    /**  Swap the contents of the specified rows.
     */
    void swap(row_index i, row_index j);

    /** Title is equal.
     */
    bool titleq(const std::string& ttl) const;

  private:
    std::string mTitle;
    std::string mXType;
    std::vector<Cell> mRows;
  };

  //=====================================  Inline methods
  inline void
  Column::addRow(const Cell& c) {
      mRows.push_back(c);
  }

  inline const char*
  Column::getTitle(void) const {
      return mTitle.c_str();
  }

  inline const Cell& 
  Column::operator[](row_index i) const {
      return mRows[i];
  }

  inline Cell& 
  Column::operator[](row_index i) {
      return mRows[i];
  }

  inline const Cell& 
  Column::refCell(row_index i) const {
      return mRows[i];
  }
  
  inline Cell& 
  Column::refCell(row_index i) {
      return mRows[i];
  }

  inline const char* 
  Column::getXType(void) const {
      return mXType.c_str();
  }

  inline bool
  Column::less(row_index i, row_index j) const {
      return mRows[i].getString() < mRows[j].getString();
  }

  inline bool
  Column::lesn(row_index i, row_index j) const {
      return mRows[i].getNumeric() < mRows[j].getNumeric();
  }

  inline const std::string&
  Column::refTitle(void) const {
      return mTitle;
  }

  inline Column::row_index
  Column::size(void) const {
      return mRows.size();
  }

  inline void
  Column::swap(row_index i, row_index j) {
      Cell t=mRows[i];
      mRows[i] = mRows[j];
      mRows[j] = t;
  }

}

#endif // !defined(TABLE_COLUMN_HH)
