//
//  Table manipulation utility
//
#include <iostream>
#include <sstream>
#include <fstream>
#include "table.hh"
#include "eval.hh"
#include "ParseLine.hh"
#include "VArg.hh"
//
#include "TCanvas.h"
#include "TH1.h"
#include <sys/times.h>
#include <limits.h>

using namespace table;
using namespace std;

void help(ostream& out);
void exec(istream& in, Table& t, eval::dict_type& d);

int debug(0);
int verbose(0);

int 
main(int argc, const char* argv[]) {
    if (argc < 2) {
        help(cerr);
	return 1;
    }
    Table t;
    eval::dict_type d;
    bool title(true), body(true), sum(false), syntax (false);
    string tablename, cmdfile, command, ofile("-"), ftype;
    for (int i=1; i<argc; i++) {
        string argi(argv[i]);
	if (argi == "-b") {
	    body = false;
	} else if (argi == "-s") {
	    sum = true;
	} else if (argi == "-h") {
	    title = false;
	} else if (argi == "--help") {
	    help(cout);
	    return 0;
	} else if (argi == "-html") {
	    ftype = "html";
	} else if (argi == "-i") {
	    if (++i < argc) cmdfile = argv[i];
	    else            syntax = true;
	} else if (argi == "-c") {
	    if (++i < argc) {
	        if (!command.empty() && command[command.size()-1] != ';') {
		    command += ";";
		}
	        command += argv[i];
	    } else {
	        syntax = true;
	    }
	} else if (argi == "-o") {
	    if (++i < argc) ofile = argv[i];
	    else            syntax = true;
	} else if (argi == "-t") {
	    if (++i < argc) tablename = argv[i];
	    else            syntax = true;
	} else if (argi == "-v") {
	    verbose = 1;
	} else if (argi == "-xml") {
	    ftype = "xml";
	} else if (argi == "--debug") {
	    if (++i < argc) debug = strtol(argv[i], 0, 0);
	    else            debug = 1;
	    t.setDebug(debug);
	    verbose = 1;
	} else if (argi == "--version") {
	    cout << "DMT table manipulation utility - $Id: tablepgm.cc,v 1.23 2006/07/20 04:22:59 jzweizig Exp $" << endl;
	    return 0;
	} else if (argi[0] == '-' && argi != "-") {
            cerr << "Undefined keyword: " << argi << endl;
	    syntax = true;
	    break;
	} else {
	    string thisfile = argi;
	    string thisname = tablename;
	    cerr << "Reading file: " << thisfile << endl;
	    try {
	        Table thistable(thisname, "");
		thistable.setDebug(debug);
		thistable.read(thisfile);
		if (thisname.empty()) thisname = thisfile;
		if (!t.size()) t=thistable;
		else           t.insertRows(t.getNRows(), thistable);
	    } catch (std::exception& e) {
	        cerr << "Caught exception reading table " << thisfile
		     << ": " << e.what() << endl;
		return 1;
	    }
	}
    }
    if (syntax) {
        help(cerr);
	return 1;
    }

    //----------------------------------  Make the table
    while (!command.empty()) {
        string::size_type l = command.find(";");
	if (debug) cout << "exec(" << command.substr(0, l) << ")" << endl;
        istringstream in(command.substr(0, l));
	exec(in, t, d);
	if (l != string::npos) command.erase(0, l+1);
	else                   command.clear();
    }

    
    if (cmdfile == "-") {
        if (debug) cout << "execfrom(cin)" << endl;
	exec(cin, t, d);
    } else if (!cmdfile.empty()) {
	if (debug) cout << "execfrom(" << cmdfile << ")" << endl;
        ifstream in(cmdfile.c_str());
	exec(in, t, d);
    }

    //----------------------------------  All done, write the table
    int flags = 0;
    if (title) flags += 1;
    if (body)  flags += 2;
    if (sum)   flags += 4;
    t.write(ofile ,flags, ftype); 
    return 0;
}

//======================================  remove blanks
void
deblank(string& s) {
    unsigned int i=0;
    while (i<s.size()) {
        if (s[i] == ' ') s.erase(i, 1);
        else if (s[i] == '\t') s.erase(i, 1);
	else i++;
    }
}

//======================================  Print user time
double tDelta(0.0);
void
usertime(void) {
    struct tms t;
    times(&t);
    double tNow(t.tms_utime/double(sysconf(_SC_CLK_TCK)));
    cerr<< "user time: " << tNow-tDelta <<  " (total: " << tNow << ")" << endl;
    tDelta = tNow;
}

//======================================  Execute a list of commands.
void 
exec(istream& in, Table& t, eval::dict_type& d) {
    ParseLine pl(in);
    if (verbose) pl.setLog(cerr);
    VArg va;
    while (pl.getLine() >= 0) {
        va.erase();
        try {
	    unsigned int nArg=pl.getCount();
	    string cmd(pl[0]);

	    //--------------------------  Add a column
	    if (cmd == "addcolumn") {
	        va << "-before" << "-eval" << "-title";
		va.scan(pl, 1);
		int inx(0);
		if (!va["-before"].empty()) inx = t.find(va["-before"]);
		else                        inx = t.size();
		t.insertColumn(inx, va["-title"]);
		if (!va["-eval"].empty()) {
		    eval evcmd(t, va["-eval"], debug);
		    evcmd.setDictionary(d);
		    int N = t.getNRows();
		    for (int i=0; i<N; i++) {
		        t[inx].refCell(i) = Cell(evcmd.evalRow(i));
		    }
		}

	    //--------------------------  Add rows from another table
	    } else if (cmd == "addrows" || cmd == "insertrows") {
	        va << "-table";
		va.scan(pl, 1);
		if (!va["-table"].empty()) {
		    Table t2(va["-table"], va["-table"]);
		    t.insertRows(t.getNRows(), t2);
		}

	    //--------------------------  Plot a histogram
	    } else if (cmd == "histogram") {
		int N = t.getNRows();
		if (!N) throw runtime_error("Empty table");

	        va << "-eval";
		va << "-file" = "tablepgm.ps";
		va << "-nbin" = "100";
		va << "-title";
		va << "-weight";
		va << "-xlog" = "false";
		va << "-xmin" = "0";
		va << "-xmax" = "0";
		va << "-ylog" = "false";
		va << "-ymin" = "0";
		va << "-ymax" = "0";
		va.scan(pl, 1);

		//----------------------  Get the histogram values
		eval evcmd(t, va["-eval"], debug);
		evcmd.setDictionary(d);
		std::vector<double> val(N);
		for (int i=0; i<N; ++i) val[i] = evcmd.evalRow_numeric(i);

		//----------------------  Get the histogram limits
		double xmin = va.getDouble("-xmin");
		double xmax = va.getDouble("-xmax");
		if (xmax <= xmin) {
		    xmin = val[0];
		    xmax = val[0];
		    for (int i=1; i<N; ++i) {
		        double x = val[i];
			if      (x < xmin) xmin = x;
			else if (x > xmax) xmax = x;
		    }
		}

		//----------------------  Book the histo
		string title = va["-title"];
		if (title.empty()) title = va["-eval"] + " histogram";
		long nbin = long(va.getDouble("-nbin"));
		TH1D histo("TblPlot", title.c_str(), nbin, xmin, xmax);

		//----------------------  Fill it
		if (va["-weight"].empty()) {
		    histo.FillN(N, &val[0], 0);
		} else {
		    eval evalwt(t, va["-weight"], debug);
		    evalwt.setDictionary(d);
		    std::vector<double> wt(N);
		    for (int i=0; i<N; ++i) wt[i] = evcmd.evalRow_numeric(i);
		    histo.FillN(N, &val[0], &wt[0]);
		}

		//----------------------  Plot it
		TCanvas c("tablepgm", "Table Histo");
		if (va.getBool("-ylog")) c.SetLogy(1);
		if (va.getBool("-xlog")) c.SetLogx(1);
		histo.DrawCopy("");
		c.Update();
		c.Print(va["-file"].c_str());

	    //--------------------------  Merge two or more columns
	    } else if (cmd == "merge") {
	        va << "-join" = " ";
	        Table::col_index inx = va.scanx(pl, 1, nArg);
		string join = va["-join"];

		string name = pl[inx++];
		if (!t.exists(name)) t.insertColumn(t.size(), name);

	        for (Table::col_index i=inx; i<nArg; ++i) {
		    Table::col_index icol = t.find(name);
		    Table::col_index jcol = t.find(pl[i]);
		    if (jcol == t.size()) {
		        throw runtime_error("merge::unrecognized column");
		    }
		    t.merge(icol, jcol, join);
		    if (icol != jcol) t.removeColumn(jcol);
		}

	    //--------------------------  Merge two tables
	    } else if (cmd == "mergetable") {
	        va << "-columns" << "-file" << "-key";
		va << "-oper"    = "+";
		va << "-ordered" = "false";
		va.scan(pl, 1);

		string file = va["-file"];
		if (file.empty()) throw runtime_error("File not specified");
		string key  = va["-key"];
		const Column& keyCol1 = t.findColumn(key);
		string cols  = va["-columns"];
		string oper  = va["-oper"];

		//----------------------  Get the table
		Table t2(file, file);
		const Column& keyCol2 = t2.findColumn(key);

		//----------------------  Loop over 
		vector<Column*> pCol1;
		vector<Column*> pCol2;
		while (!cols.empty()) {
		    string::size_type inx = cols.find(",");
		    string name;
		    if (inx == string::npos) {
		        name = cols;
			cols.clear();
		    } else {
		        name = cols.substr(0,inx);
			cols.erase(0,inx+1);
		    }
		    pCol1.push_back(&t.findColumn(name));
		    pCol2.push_back(&t2.findColumn(name));
		}
		Column::row_index nColMod = pCol1.size();

		Column::row_index n1 = keyCol1.size();
		Column::row_index n2 = keyCol2.size();
		for (Column::row_index i=0; i<n1; ++i) {
		    for (Column::row_index j=0; j<n2; ++j) {
		        if (keyCol1[i] == keyCol2[j]) {
			    for (Column::row_index k=0; k<nColMod; ++k) {
			        if (oper == "|") {
				    (*pCol1[k])[i] |= (*pCol2[k])[j];
				} else {
				    (*pCol1[k])[i] += (*pCol2[k])[j];
				}
			    }
			}
		    }
		}

	    //--------------------------  Rmove one or more rows or columns
	    } else if (cmd == "remove") {
	        for (unsigned int i=1; i<nArg; ++i) {
		    string argi = pl[i];
		    if (argi == "-column") {
		        Table::col_index inx = t.find(pl[++i]);
			if (inx < t.size()) t.removeColumn(inx);
			else                cerr << "Remove: Column " << pl[i]
						 << " not found." << endl;
		    } else if (argi == "-rows") {
		        eval evcmd(t, pl[++i], debug);
			evcmd.setDictionary(d);
			int N = t.getNRows();
			vector<bool> mask(N);
			for (int i=0; i<N; ++i) {
			    mask[i] = !evcmd.evalRow_bool(i);
			}
			t.selectRows(mask);
			if (verbose) {
			    cerr << "Removed " << N-t.getNRows() << " rows, "
				 << t.getNRows() << " remain." << endl;
			}
		    } else {
		        cerr << "Unrecognized key: " << argi << endl;
			break;
		    }
		}

	    //--------------------------  Replace values in a column
	    } else if (cmd == "replace") {
	        va <<  "-column" << "-rows" << "-with";
		va.scan(pl,1);

		eval evwith(t, va["-with"], debug);
		evwith.setDictionary(d);

		Column& col(t.findColumn(va["-column"]));
		Column::row_index N = t.getNRows();

		if (va["-rows"].empty()) {
		    for (Column::row_index i=0; i<N; ++i) {
		        col.refCell(i) = Cell(evwith.evalRow(i));
		    }
		} else {
		    eval evsel(t, va["-rows"], debug);
		    evsel.setDictionary(d);
		    for (Column::row_index i=0; i<N; ++i) {
		        if (evsel.evalRow_bool(i)) {
		            col.refCell(i) = Cell(evwith.evalRow(i));
			}
		    }
		}

	    //--------------------------  Set column attributes
	    } else if (cmd == "setcolumn") {
		Column& col(t.findColumn(pl[1]));

		va << "-title" << "-format" << "-extern" << "-justify";
		va.scan(pl, 2, nArg);

		if (!va["-title"].empty())   col.setTitle(va["-title"]);
		if (!va["-extern"].empty())  col.setXType(va["-extern"]);
		if (!va["-format"].empty())  col.setTitle(va["-format"]);
		if (!va["-justify"].empty()) col.setTitle(va["-justify"]);

	    //--------------------------  Sort rows
	    } else if (cmd == "sort") {
	        bool   decrease = false;
		Table::sort_enum sType = Table::kAuto;
	        for (unsigned int i=1; i<nArg; ++i) {
		    string argi = pl[i];
		    if (argi == "-down") {
		        decrease = true;
		    } else if (argi == "-key") {
		        Table::col_index inx = t.find(pl[++i]);
			if (inx < t.size()) t.sort(inx, decrease, sType);
			else                cerr << "Sort: key " << pl[i]
						 << " not found." << endl;
		    } else if (argi == "-up") {
		        decrease = false;
		    } else if (argi == "-sort-type") {
		        argi = pl[++i];
			if (argi == "auto") {
			    sType = Table::kAuto;
			} else if (argi == "bubble") {
			    sType = Table::kBubble;
			} else if (argi == "merge") {
			    sType = Table::kMerge;
			} else {
			    cerr << "Unrecognized sort type: " << argi << endl;
			}			
		    } else {
		        cerr << "Unrecognized sort option: " << argi << endl;
			break;
		    }
		}

	    //--------------------------  Write out the table
	    } else if (cmd == "write") {
	        va << "-html" << "-xml" << "-noheader" <<  "-summary";
		va << "-nobody" << "-file" = "-";
		va.scan(pl, 1);

		string ftype;
		if (va.getBool("-html")) ftype = "html";
		if (va.getBool("-xml"))  ftype = "xml";

		int  flags=3;
		if (va.getBool("-nobody"))   flags &= ~1;
		if (va.getBool("-noheader")) flags &= ~2;
		if (va.getBool("-summary"))  flags |= 4;

		t.write(va["-file"], flags, ftype);

	    //--------------------------  Define a symbol or error
	    } else {
	        unsigned int inx=cmd.find("=");
		if (!inx || inx >= cmd.size()-1) {
		    cerr << "Unrecognized command: " << cmd << endl;
		} else {
		    string sym = cmd.substr(0,inx);
		    inx++;
		    string val = cmd.substr(inx, cmd.size()-inx);
		    deblank(sym);
		    deblank(val);
		    d[sym]=val;
		    if (debug) cerr << "Symbol: " << sym << " set to " 
				    << val << endl;
		}
	    }

	//------------------------------  Error in a command.
	} catch (exception& e) {
	    cerr << "Exception: " << e.what() << ", caught at line: " 
		 << pl.getLineNumber() << endl;
	}
	if (verbose) usertime();
    }
}

void
help(ostream& out) {
    out << "Syntax: " << endl;
    out << "  tablepgm [options] <file1> [... <fileN>]" << endl;
    out << "Processing options: " << endl;
    out << "  -c <cmd>    Execute a list of commands" << endl;
    out << "  -i <cfile>  Execute commands in the specified file" << endl;
    out << "  -t <name>   Table name" << endl;
    out << "  --debug [n] Set the debug level." << endl;
    out << "  --help      Print this message and exit." << endl;
    out << "  --version   Print cvs version ID and exit." << endl; 
    out << "Output options: " << endl;
    out << "  -b          Print table body only" << endl;
    out << "  -html       Write table as html" << endl;
    out << "  -o <ofile>  output file name" << endl;
    out << "  -s          Print sum of all numeric columns" << endl;
    out << "  -xml        Write table as xml" << endl;
    out << "Commands: " << endl;
    out << "addcolumn addrows histogram merge remove replace setcolumn "
	<< "sort write" << endl;
}
